package amiga;

public class Trigonometry {

	/**
	 * value that is equivalent to 360 degrees
	 */
	static final int MAX_ANGLE = 1 << 16;
	static final int ANGLE_MASK = (1 << 16) - 1;

	/** 65536, 0x10000 */
	static final int ANGLE_360_DEGREES = MAX_ANGLE;
	/** 49152, 0xC000 */
	static final int ANGLE_270_DEGREES = 3 * MAX_ANGLE / 4;
	/** 32768, 0x8000 */
	static final int ANGLE_180_DEGREES = MAX_ANGLE / 2;
	/** 16384, 0x4000 */
	static final int ANGLE_90_DEGREES = MAX_ANGLE / 4;
	/** 16383, 0x3FFF */
	static final int ANGLE_90_DEGREE_MASK = ANGLE_90_DEGREES - 1;
	/** 16382, 0x3FFE */
	static final int ANGLE_90_DEGREE_MASK_EVEN = ANGLE_90_DEGREES - 2;
	/** 49152, 0xC000 */
	static final int ANGLE_90_DEGREE_STEP_MASK = ANGLE_90_DEGREES + ANGLE_180_DEGREES;
	/** 8192, 0x2000 */
	static final int ANGLE_45_DEGREES = MAX_ANGLE / 8;
	/** 7.03 degrees, 1280, 0x500 */
	static final int ANGLE_7_DEGREES = 0x500;
	/** 256, 0x100 */
	static final int ANGLE_1p43_DEGREES = MAX_ANGLE / 256;
	static final int ANGLE_0_DEGREES = 0;

	/** 0.707 * 256 */
	static final int SIN_45_FP8 = 0xB5;

	// perspective_table	equ	cos_table+516
	// perspective_table2	equ	perspective_table+4096
	private static final int PERSPECTIVE_TABLE = 516;
	private static final int PERSPECTIVE_TABLE_2 = PERSPECTIVE_TABLE + 4096;

	/** cos table, 256 values */
	static char[] w_cos_table = {
			(char)0xffff, (char)0xfffe, (char)0xfffb, (char)0xfff4, (char)0xffec, (char)0xffe1, (char)0xffd3, (char)0xffc3, (char)0xffb1, (char)0xff9c, (char)0xff84, (char)0xff6a, (char)0xff4e, (char)0xff2f, (char)0xff0e, (char)0xfeea,
			(char)0xfec4, (char)0xfe9b, (char)0xfe70, (char)0xfe43, (char)0xfe13, (char)0xfde0, (char)0xfdab, (char)0xfd74, (char)0xfd3a, (char)0xfcfe, (char)0xfcbf, (char)0xfc7e, (char)0xfc3b, (char)0xfbf5, (char)0xfbac, (char)0xfb61,
			(char)0xfb14, (char)0xfac5, (char)0xfa73, (char)0xfa1e, (char)0xf9c7, (char)0xf96e, (char)0xf912, (char)0xf8b4, (char)0xf853, (char)0xf7f1, (char)0xf78b, (char)0xf724, (char)0xf6ba, (char)0xf64d, (char)0xf5de, (char)0xf56d,
			(char)0xf4fa, (char)0xf484, (char)0xf40b, (char)0xf391, (char)0xf314, (char)0xf294, (char)0xf213, (char)0xf18f, (char)0xf109, (char)0xf080, (char)0xeff5, (char)0xef68, (char)0xeed8, (char)0xee46, (char)0xedb2, (char)0xed1c,
			(char)0xec83, (char)0xebe8, (char)0xeb4b, (char)0xeaab, (char)0xea09, (char)0xe965, (char)0xe8bf, (char)0xe816, (char)0xe76b, (char)0xe6be, (char)0xe60f, (char)0xe55e, (char)0xe4aa, (char)0xe3f4, (char)0xe33c, (char)0xe282,
			(char)0xe1c5, (char)0xe106, (char)0xe046, (char)0xdf83, (char)0xdebe, (char)0xddf6, (char)0xdd2d, (char)0xdc61, (char)0xdb94, (char)0xdac4, (char)0xd9f2, (char)0xd91e, (char)0xd848, (char)0xd770, (char)0xd695, (char)0xd5b9,
			(char)0xd4db, (char)0xd3fa, (char)0xd318, (char)0xd233, (char)0xd14d, (char)0xd064, (char)0xcf7a, (char)0xce8d, (char)0xcd9f, (char)0xccae, (char)0xcbbb, (char)0xcac7, (char)0xc9d1, (char)0xc8d8, (char)0xc7de, (char)0xc6e2,
			(char)0xc5e4, (char)0xc4e3, (char)0xc3e2, (char)0xc2de, (char)0xc1d8, (char)0xc0d0, (char)0xbfc7, (char)0xbebc, (char)0xbdae, (char)0xbca0, (char)0xbb8f, (char)0xba7c, (char)0xb968, (char)0xb852, (char)0xb73a, (char)0xb620,
			(char)0xb504, (char)0xb3e7, (char)0xb2c8, (char)0xb1a8, (char)0xb085, (char)0xaf61, (char)0xae3b, (char)0xad14, (char)0xabeb, (char)0xaac0, (char)0xa994, (char)0xa866, (char)0xa736, (char)0xa605, (char)0xa4d2, (char)0xa39d,
			(char)0xa267, (char)0xa12f, (char)0x9ff6, (char)0x9ebc, (char)0x9d7f, (char)0x9c42, (char)0x9b02, (char)0x99c2, (char)0x987f, (char)0x973c, (char)0x95f6, (char)0x94b0, (char)0x9368, (char)0x921e, (char)0x90d3, (char)0x8f87,
			(char)0x8e39, (char)0x8cea, (char)0x8b9a, (char)0x8a48, (char)0x88f5, (char)0x87a1, (char)0x864b, (char)0x84f4, (char)0x839c, (char)0x8242, (char)0x80e7, (char)0x7f8b, (char)0x7e2e, (char)0x7cd0, (char)0x7b70, (char)0x7a0f,
			(char)0x78ad, (char)0x774a, (char)0x75e5, (char)0x7480, (char)0x7319, (char)0x71b1, (char)0x7049, (char)0x6edf, (char)0x6d74, (char)0x6c08, (char)0x6a9b, (char)0x692d, (char)0x67bd, (char)0x664d, (char)0x64dc, (char)0x636a,
			(char)0x61f7, (char)0x6083, (char)0x5f0e, (char)0x5d98, (char)0x5c22, (char)0x5aaa, (char)0x5931, (char)0x57b8, (char)0x563e, (char)0x54c3, (char)0x5347, (char)0x51ca, (char)0x504d, (char)0x4ecf, (char)0x4d50, (char)0x4bd0,
			(char)0x4a50, (char)0x48ce, (char)0x474d, (char)0x45ca, (char)0x4447, (char)0x42c3, (char)0x413e, (char)0x3fb9, (char)0x3e33, (char)0x3cad, (char)0x3b26, (char)0x399f, (char)0x3817, (char)0x368e, (char)0x3505, (char)0x337b,
			(char)0x31f1, (char)0x3066, (char)0x2edb, (char)0x2d50, (char)0x2bc4, (char)0x2a37, (char)0x28aa, (char)0x271d, (char)0x2590, (char)0x2402, (char)0x2273, (char)0x20e5, (char)0x1f56, (char)0x1dc7, (char)0x1c37, (char)0x1aa7,
			(char)0x1917, (char)0x1787, (char)0x15f6, (char)0x1466, (char)0x12d5, (char)0x1144, (char)0xfb2, (char)0xe21, (char)0xc8f, (char)0xafe, (char)0x96c, (char)0x7da, (char)0x648, (char)0x4b6, (char)0x324, (char)0x192,
			(char)0x0, (char)0x0 };
	static short[] w_perspective_table = {
			// 0
			(short)0x0, (short)0x5, (short)0xa, (short)0xf, (short)0x14, (short)0x19, (short)0x1f, (short)0x24, (short)0x29, (short)0x2e, (short)0x33, (short)0x38, (short)0x3d, (short)0x42, (short)0x47, (short)0x4c,
			(short)0x51, (short)0x57, (short)0x5c, (short)0x61, (short)0x66, (short)0x6b, (short)0x70, (short)0x75, (short)0x7a, (short)0x7f, (short)0x84, (short)0x8a, (short)0x8f, (short)0x94, (short)0x99, (short)0x9e,
			(short)0xa3, (short)0xa8, (short)0xad, (short)0xb2, (short)0xb7, (short)0xbc, (short)0xc2, (short)0xc7, (short)0xcc, (short)0xd1, (short)0xd6, (short)0xdb, (short)0xe0, (short)0xe5, (short)0xea, (short)0xef,
			(short)0xf4, (short)0xfa, (short)0xff, (short)0x104, (short)0x109, (short)0x10e, (short)0x113, (short)0x118, (short)0x11d, (short)0x122, (short)0x127, (short)0x12c, (short)0x131, (short)0x137, (short)0x13c, (short)0x141,
			(short)0x146, (short)0x14b, (short)0x150, (short)0x155, (short)0x15a, (short)0x15f, (short)0x164, (short)0x169, (short)0x16f, (short)0x174, (short)0x179, (short)0x17e, (short)0x183, (short)0x188, (short)0x18d, (short)0x192,
			(short)0x197, (short)0x19c, (short)0x1a1, (short)0x1a6, (short)0x1ac, (short)0x1b1, (short)0x1b6, (short)0x1bb, (short)0x1c0, (short)0x1c5, (short)0x1ca, (short)0x1cf, (short)0x1d4, (short)0x1d9, (short)0x1de, (short)0x1e3,
			(short)0x1e9, (short)0x1ee, (short)0x1f3, (short)0x1f8, (short)0x1fd, (short)0x202, (short)0x207, (short)0x20c, (short)0x211, (short)0x216, (short)0x21b, (short)0x220, (short)0x226, (short)0x22b, (short)0x230, (short)0x235,
			(short)0x23a, (short)0x23f, (short)0x244, (short)0x249, (short)0x24e, (short)0x253, (short)0x258, (short)0x25d, (short)0x262, (short)0x268, (short)0x26d, (short)0x272, (short)0x277, (short)0x27c, (short)0x281, (short)0x286,
			(short)0x28b, (short)0x290, (short)0x295, (short)0x29a, (short)0x29f, (short)0x2a4, (short)0x2a9, (short)0x2af, (short)0x2b4, (short)0x2b9, (short)0x2be, (short)0x2c3, (short)0x2c8, (short)0x2cd, (short)0x2d2, (short)0x2d7,
			(short)0x2dc, (short)0x2e1, (short)0x2e6, (short)0x2eb, (short)0x2f0, (short)0x2f6, (short)0x2fb, (short)0x300, (short)0x305, (short)0x30a, (short)0x30f, (short)0x314, (short)0x319, (short)0x31e, (short)0x323, (short)0x328,
			(short)0x32d, (short)0x332, (short)0x337, (short)0x33c, (short)0x341, (short)0x347, (short)0x34c, (short)0x351, (short)0x356, (short)0x35b, (short)0x360, (short)0x365, (short)0x36a, (short)0x36f, (short)0x374, (short)0x379,
			(short)0x37e, (short)0x383, (short)0x388, (short)0x38d, (short)0x392, (short)0x397, (short)0x39c, (short)0x3a2, (short)0x3a7, (short)0x3ac, (short)0x3b1, (short)0x3b6, (short)0x3bb, (short)0x3c0, (short)0x3c5, (short)0x3ca,
			(short)0x3cf, (short)0x3d4, (short)0x3d9, (short)0x3de, (short)0x3e3, (short)0x3e8, (short)0x3ed, (short)0x3f2, (short)0x3f7, (short)0x3fc, (short)0x401, (short)0x407, (short)0x40c, (short)0x411, (short)0x416, (short)0x41b,
			(short)0x420, (short)0x425, (short)0x42a, (short)0x42f, (short)0x434, (short)0x439, (short)0x43e, (short)0x443, (short)0x448, (short)0x44d, (short)0x452, (short)0x457, (short)0x45c, (short)0x461, (short)0x466, (short)0x46b,
			(short)0x470, (short)0x475, (short)0x47a, (short)0x47f, (short)0x484, (short)0x489, (short)0x48e, (short)0x494, (short)0x499, (short)0x49e, (short)0x4a3, (short)0x4a8, (short)0x4ad, (short)0x4b2, (short)0x4b7, (short)0x4bc,
			(short)0x4c1, (short)0x4c6, (short)0x4cb, (short)0x4d0, (short)0x4d5, (short)0x4da, (short)0x4df, (short)0x4e4, (short)0x4e9, (short)0x4ee, (short)0x4f3, (short)0x4f8, (short)0x4fd, (short)0x502, (short)0x507, (short)0x50c,
			// 16
			(short)0x511, (short)0x516, (short)0x51b, (short)0x520, (short)0x525, (short)0x52a, (short)0x52f, (short)0x534, (short)0x539, (short)0x53e, (short)0x543, (short)0x548, (short)0x54d, (short)0x552,	(short)0x557, (short)0x55c,
			(short)0x561, (short)0x566, (short)0x56b, (short)0x570, (short)0x575, (short)0x57a, (short)0x57f, (short)0x584, (short)0x589, (short)0x58e, (short)0x593, (short)0x598, (short)0x59d, (short)0x5a2, (short)0x5a7, (short)0x5ac,
			(short)0x5b1, (short)0x5b6, (short)0x5bb, (short)0x5c0, (short)0x5c5, (short)0x5ca, (short)0x5cf, (short)0x5d4, (short)0x5d9, (short)0x5de, (short)0x5e3, (short)0x5e8, (short)0x5ed, (short)0x5f2, (short)0x5f7, (short)0x5fc,
			(short)0x601, (short)0x606, (short)0x60b, (short)0x610, (short)0x615, (short)0x61a, (short)0x61f, (short)0x624, (short)0x629, (short)0x62e, (short)0x633, (short)0x638, (short)0x63d, (short)0x642, (short)0x647, (short)0x64c,
			(short)0x651, (short)0x656, (short)0x65b, (short)0x660, (short)0x665, (short)0x66a, (short)0x66e, (short)0x673, (short)0x678, (short)0x67d, (short)0x682, (short)0x687, (short)0x68c, (short)0x691, (short)0x696, (short)0x69b,
			(short)0x6a0, (short)0x6a5, (short)0x6aa, (short)0x6af, (short)0x6b4, (short)0x6b9, (short)0x6be, (short)0x6c3, (short)0x6c8, (short)0x6cd, (short)0x6d2, (short)0x6d7, (short)0x6dc, (short)0x6e1, (short)0x6e5, (short)0x6ea,
			(short)0x6ef, (short)0x6f4, (short)0x6f9, (short)0x6fe, (short)0x703, (short)0x708, (short)0x70d, (short)0x712, (short)0x717, (short)0x71c, (short)0x721, (short)0x726, (short)0x72b, (short)0x730, (short)0x735, (short)0x739,
			(short)0x73e, (short)0x743, (short)0x748, (short)0x74d, (short)0x752, (short)0x757, (short)0x75c, (short)0x761, (short)0x766, (short)0x76b, (short)0x770, (short)0x775, (short)0x77a, (short)0x77e, (short)0x783, (short)0x788,
			(short)0x78d, (short)0x792, (short)0x797, (short)0x79c, (short)0x7a1, (short)0x7a6, (short)0x7ab, (short)0x7b0, (short)0x7b5, (short)0x7b9, (short)0x7be, (short)0x7c3, (short)0x7c8, (short)0x7cd, (short)0x7d2, (short)0x7d7,
			(short)0x7dc, (short)0x7e1, (short)0x7e6, (short)0x7eb, (short)0x7ef, (short)0x7f4, (short)0x7f9, (short)0x7fe, (short)0x803, (short)0x808, (short)0x80d, (short)0x812, (short)0x817, (short)0x81c, (short)0x820, (short)0x825,
			(short)0x82a, (short)0x82f, (short)0x834, (short)0x839, (short)0x83e, (short)0x843, (short)0x848, (short)0x84c, (short)0x851, (short)0x856, (short)0x85b, (short)0x860, (short)0x865, (short)0x86a, (short)0x86f, (short)0x873,
			(short)0x878, (short)0x87d, (short)0x882, (short)0x887, (short)0x88c, (short)0x891, (short)0x896, (short)0x89a, (short)0x89f, (short)0x8a4, (short)0x8a9, (short)0x8ae, (short)0x8b3, (short)0x8b8, (short)0x8bd, (short)0x8c1,
			(short)0x8c6, (short)0x8cb, (short)0x8d0, (short)0x8d5, (short)0x8da, (short)0x8df, (short)0x8e3, (short)0x8e8, (short)0x8ed, (short)0x8f2, (short)0x8f7, (short)0x8fc, (short)0x901, (short)0x905, (short)0x90a, (short)0x90f,
			(short)0x914, (short)0x919, (short)0x91e, (short)0x922, (short)0x927, (short)0x92c, (short)0x931, (short)0x936, (short)0x93b, (short)0x93f, (short)0x944, (short)0x949, (short)0x94e, (short)0x953, (short)0x958, (short)0x95c,
			(short)0x961, (short)0x966, (short)0x96b, (short)0x970, (short)0x975, (short)0x979, (short)0x97e, (short)0x983, (short)0x988, (short)0x98d, (short)0x992, (short)0x996, (short)0x99b, (short)0x9a0, (short)0x9a5, (short)0x9aa,
			(short)0x9ae, (short)0x9b3, (short)0x9b8, (short)0x9bd, (short)0x9c2, (short)0x9c6, (short)0x9cb, (short)0x9d0, (short)0x9d5, (short)0x9da, (short)0x9de, (short)0x9e3, (short)0x9e8, (short)0x9ed, (short)0x9f2, (short)0x9f6,
			// 32
			(short)0x9fb, (short)0xa00, (short)0xa05, (short)0xa0a, (short)0xa0e, (short)0xa13, (short)0xa18, (short)0xa1d, (short)0xa22, (short)0xa26, (short)0xa2b, (short)0xa30, (short)0xa35, (short)0xa39, (short)0xa3e, (short)0xa43,
			(short)0xa48, (short)0xa4d, (short)0xa51, (short)0xa56, (short)0xa5b, (short)0xa60, (short)0xa64, (short)0xa69, (short)0xa6e, (short)0xa73, (short)0xa77, (short)0xa7c, (short)0xa81, (short)0xa86, (short)0xa8b, (short)0xa8f,
			(short)0xa94, (short)0xa99, (short)0xa9e, (short)0xaa2, (short)0xaa7, (short)0xaac, (short)0xab1, (short)0xab5, (short)0xaba, (short)0xabf, (short)0xac4, (short)0xac8, (short)0xacd, (short)0xad2, (short)0xad7, (short)0xadb,
			(short)0xae0, (short)0xae5, (short)0xae9, (short)0xaee, (short)0xaf3, (short)0xaf8, (short)0xafc, (short)0xb01, (short)0xb06, (short)0xb0b, (short)0xb0f, (short)0xb14, (short)0xb19, (short)0xb1e, (short)0xb22, (short)0xb27,
			(short)0xb2c, (short)0xb30, (short)0xb35, (short)0xb3a, (short)0xb3f, (short)0xb43, (short)0xb48, (short)0xb4d, (short)0xb51, (short)0xb56, (short)0xb5b, (short)0xb60, (short)0xb64, (short)0xb69, (short)0xb6e, (short)0xb72,
			(short)0xb77, (short)0xb7c, (short)0xb80, (short)0xb85, (short)0xb8a, (short)0xb8f, (short)0xb93, (short)0xb98, (short)0xb9d, (short)0xba1, (short)0xba6, (short)0xbab, (short)0xbaf, (short)0xbb4, (short)0xbb9, (short)0xbbd,
			(short)0xbc2, (short)0xbc7, (short)0xbcb, (short)0xbd0, (short)0xbd5, (short)0xbd9, (short)0xbde, (short)0xbe3, (short)0xbe7, (short)0xbec, (short)0xbf1, (short)0xbf5, (short)0xbfa, (short)0xbff, (short)0xc03, (short)0xc08,
			(short)0xc0d, (short)0xc11, (short)0xc16, (short)0xc1b, (short)0xc1f, (short)0xc24, (short)0xc29, (short)0xc2d, (short)0xc32, (short)0xc37, (short)0xc3b, (short)0xc40, (short)0xc45, (short)0xc49, (short)0xc4e, (short)0xc53,
			(short)0xc57, (short)0xc5c, (short)0xc60, (short)0xc65, (short)0xc6a, (short)0xc6e, (short)0xc73, (short)0xc78, (short)0xc7c, (short)0xc81, (short)0xc86, (short)0xc8a, (short)0xc8f, (short)0xc93, (short)0xc98, (short)0xc9d,
			(short)0xca1, (short)0xca6, (short)0xcab, (short)0xcaf, (short)0xcb4, (short)0xcb8, (short)0xcbd, (short)0xcc2, (short)0xcc6, (short)0xccb, (short)0xccf, (short)0xcd4, (short)0xcd9, (short)0xcdd, (short)0xce2, (short)0xce6,
			(short)0xceb, (short)0xcf0, (short)0xcf4, (short)0xcf9, (short)0xcfd, (short)0xd02, (short)0xd07, (short)0xd0b, (short)0xd10, (short)0xd14, (short)0xd19, (short)0xd1e, (short)0xd22, (short)0xd27, (short)0xd2b, (short)0xd30,
			(short)0xd34, (short)0xd39, (short)0xd3e, (short)0xd42, (short)0xd47, (short)0xd4b, (short)0xd50, (short)0xd54, (short)0xd59, (short)0xd5e, (short)0xd62, (short)0xd67, (short)0xd6b, (short)0xd70, (short)0xd74, (short)0xd79,
			(short)0xd7d, (short)0xd82, (short)0xd87, (short)0xd8b, (short)0xd90, (short)0xd94, (short)0xd99, (short)0xd9d, (short)0xda2, (short)0xda6, (short)0xdab, (short)0xdaf, (short)0xdb4, (short)0xdb9, (short)0xdbd, (short)0xdc2,
			(short)0xdc6, (short)0xdcb, (short)0xdcf, (short)0xdd4, (short)0xdd8, (short)0xddd, (short)0xde1, (short)0xde6, (short)0xdea, (short)0xdef, (short)0xdf3, (short)0xdf8, (short)0xdfc, (short)0xe01, (short)0xe05, (short)0xe0a,
			(short)0xe0f, (short)0xe13, (short)0xe18, (short)0xe1c, (short)0xe21, (short)0xe25, (short)0xe2a, (short)0xe2e, (short)0xe33, (short)0xe37, (short)0xe3c, (short)0xe40, (short)0xe45, (short)0xe49, (short)0xe4e, (short)0xe52,
			(short)0xe56, (short)0xe5b, (short)0xe5f, (short)0xe64, (short)0xe68, (short)0xe6d, (short)0xe71, (short)0xe76, (short)0xe7a, (short)0xe7f, (short)0xe83, (short)0xe88, (short)0xe8c, (short)0xe91, (short)0xe95, (short)0xe9a,
			// 48
			(short)0xe9e, (short)0xea3, (short)0xea7, (short)0xeac, (short)0xeb0, (short)0xeb4, (short)0xeb9, (short)0xebd, (short)0xec2, (short)0xec6, (short)0xecb, (short)0xecf, (short)0xed4, (short)0xed8, (short)0xedc, (short)0xee1,
			(short)0xee5, (short)0xeea, (short)0xeee, (short)0xef3, (short)0xef7, (short)0xefc, (short)0xf00, (short)0xf04, (short)0xf09, (short)0xf0d, (short)0xf12, (short)0xf16, (short)0xf1b, (short)0xf1f, (short)0xf23, (short)0xf28,
			(short)0xf2c, (short)0xf31, (short)0xf35, (short)0xf3a, (short)0xf3e, (short)0xf42, (short)0xf47, (short)0xf4b, (short)0xf50, (short)0xf54, (short)0xf58, (short)0xf5d, (short)0xf61, (short)0xf66, (short)0xf6a, (short)0xf6e,
			(short)0xf73, (short)0xf77, (short)0xf7c, (short)0xf80, (short)0xf84, (short)0xf89, (short)0xf8d, (short)0xf91, (short)0xf96, (short)0xf9a, (short)0xf9f, (short)0xfa3, (short)0xfa7, (short)0xfac, (short)0xfb0, (short)0xfb5,
			(short)0xfb9, (short)0xfbd, (short)0xfc2, (short)0xfc6, (short)0xfca, (short)0xfcf, (short)0xfd3, (short)0xfd7, (short)0xfdc, (short)0xfe0, (short)0xfe5, (short)0xfe9, (short)0xfed, (short)0xff2, (short)0xff6, (short)0xffa,
			(short)0xfff, (short)0x1003, (short)0x1007, (short)0x100c, (short)0x1010, (short)0x1014, (short)0x1019, (short)0x101d, (short)0x1021, (short)0x1026, (short)0x102a, (short)0x102e, (short)0x1033, (short)0x1037, (short)0x103b, (short)0x1040,
			(short)0x1044, (short)0x1048, (short)0x104d, (short)0x1051, (short)0x1055, (short)0x105a, (short)0x105e, (short)0x1062, (short)0x1067, (short)0x106b, (short)0x106f, (short)0x1073, (short)0x1078, (short)0x107c, (short)0x1080, (short)0x1085,
			(short)0x1089, (short)0x108d, (short)0x1092, (short)0x1096, (short)0x109a, (short)0x109e, (short)0x10a3, (short)0x10a7, (short)0x10ab, (short)0x10b0, (short)0x10b4, (short)0x10b8, (short)0x10bc, (short)0x10c1, (short)0x10c5, (short)0x10c9,
			(short)0x10ce, (short)0x10d2, (short)0x10d6, (short)0x10da, (short)0x10df, (short)0x10e3, (short)0x10e7, (short)0x10eb, (short)0x10f0, (short)0x10f4, (short)0x10f8, (short)0x10fd, (short)0x1101, (short)0x1105, (short)0x1109, (short)0x110e,
			(short)0x1112, (short)0x1116, (short)0x111a, (short)0x111f, (short)0x1123, (short)0x1127, (short)0x112b, (short)0x1130, (short)0x1134, (short)0x1138, (short)0x113c, (short)0x1140, (short)0x1145, (short)0x1149, (short)0x114d, (short)0x1151,
			(short)0x1156, (short)0x115a, (short)0x115e, (short)0x1162, (short)0x1166, (short)0x116b, (short)0x116f, (short)0x1173, (short)0x1177, (short)0x117c, (short)0x1180, (short)0x1184, (short)0x1188, (short)0x118c, (short)0x1191, (short)0x1195,
			(short)0x1199, (short)0x119d, (short)0x11a1, (short)0x11a6, (short)0x11aa, (short)0x11ae, (short)0x11b2, (short)0x11b6, (short)0x11bb, (short)0x11bf, (short)0x11c3, (short)0x11c7, (short)0x11cb, (short)0x11cf, (short)0x11d4, (short)0x11d8,
			(short)0x11dc, (short)0x11e0, (short)0x11e4, (short)0x11e9, (short)0x11ed, (short)0x11f1, (short)0x11f5, (short)0x11f9, (short)0x11fd, (short)0x1202, (short)0x1206, (short)0x120a, (short)0x120e, (short)0x1212, (short)0x1216, (short)0x121a,
			(short)0x121f, (short)0x1223, (short)0x1227, (short)0x122b, (short)0x122f, (short)0x1233, (short)0x1237, (short)0x123c, (short)0x1240, (short)0x1244, (short)0x1248, (short)0x124c, (short)0x1250, (short)0x1254, (short)0x1259, (short)0x125d,
			(short)0x1261, (short)0x1265, (short)0x1269, (short)0x126d, (short)0x1271, (short)0x1275, (short)0x127a, (short)0x127e, (short)0x1282, (short)0x1286, (short)0x128a, (short)0x128e, (short)0x1292, (short)0x1296, (short)0x129a, (short)0x129f,
			(short)0x12a3, (short)0x12a7, (short)0x12ab, (short)0x12af, (short)0x12b3, (short)0x12b7, (short)0x12bb, (short)0x12bf, (short)0x12c3, (short)0x12c7, (short)0x12cc, (short)0x12d0, (short)0x12d4, (short)0x12d8, (short)0x12dc, (short)0x12e0,
			// 64
			(short)0x12e4, (short)0x12e8, (short)0x12ec, (short)0x12f0, (short)0x12f4, (short)0x12f8, (short)0x12fc, (short)0x1301, (short)0x1305, (short)0x1309, (short)0x130d, (short)0x1311, (short)0x1315, (short)0x1319, (short)0x131d, (short)0x1321,
			(short)0x1325, (short)0x1329, (short)0x132d, (short)0x1331, (short)0x1335, (short)0x1339, (short)0x133d, (short)0x1341, (short)0x1345, (short)0x1349, (short)0x134d, (short)0x1351, (short)0x1355, (short)0x135a, (short)0x135e, (short)0x1362,
			(short)0x1366, (short)0x136a, (short)0x136e, (short)0x1372, (short)0x1376, (short)0x137a, (short)0x137e, (short)0x1382, (short)0x1386, (short)0x138a, (short)0x138e, (short)0x1392, (short)0x1396, (short)0x139a, (short)0x139e, (short)0x13a2,
			(short)0x13a6, (short)0x13aa, (short)0x13ae, (short)0x13b2, (short)0x13b6, (short)0x13ba, (short)0x13be, (short)0x13c2, (short)0x13c6, (short)0x13ca, (short)0x13ce, (short)0x13d2, (short)0x13d6, (short)0x13da, (short)0x13de, (short)0x13e2,
			(short)0x13e6, (short)0x13e9, (short)0x13ed, (short)0x13f1, (short)0x13f5, (short)0x13f9, (short)0x13fd, (short)0x1401, (short)0x1405, (short)0x1409, (short)0x140d, (short)0x1411, (short)0x1415, (short)0x1419, (short)0x141d, (short)0x1421,
			(short)0x1425, (short)0x1429, (short)0x142d, (short)0x1431, (short)0x1435, (short)0x1439, (short)0x143d, (short)0x1440, (short)0x1444, (short)0x1448, (short)0x144c, (short)0x1450, (short)0x1454, (short)0x1458, (short)0x145c, (short)0x1460,
			(short)0x1464, (short)0x1468, (short)0x146c, (short)0x1470, (short)0x1473, (short)0x1477, (short)0x147b, (short)0x147f, (short)0x1483, (short)0x1487, (short)0x148b, (short)0x148f, (short)0x1493, (short)0x1497, (short)0x149b, (short)0x149e,
			(short)0x14a2, (short)0x14a6, (short)0x14aa, (short)0x14ae, (short)0x14b2, (short)0x14b6, (short)0x14ba, (short)0x14be, (short)0x14c1, (short)0x14c5, (short)0x14c9, (short)0x14cd, (short)0x14d1, (short)0x14d5, (short)0x14d9, (short)0x14dd,
			(short)0x14e0, (short)0x14e4, (short)0x14e8, (short)0x14ec, (short)0x14f0, (short)0x14f4, (short)0x14f8, (short)0x14fb, (short)0x14ff, (short)0x1503, (short)0x1507, (short)0x150b, (short)0x150f, (short)0x1513, (short)0x1516, (short)0x151a,
			(short)0x151e, (short)0x1522, (short)0x1526, (short)0x152a, (short)0x152d, (short)0x1531, (short)0x1535, (short)0x1539, (short)0x153d, (short)0x1541, (short)0x1544, (short)0x1548, (short)0x154c, (short)0x1550, (short)0x1554, (short)0x1558,
			(short)0x155b, (short)0x155f, (short)0x1563, (short)0x1567, (short)0x156b, (short)0x156e, (short)0x1572, (short)0x1576, (short)0x157a, (short)0x157e, (short)0x1581, (short)0x1585, (short)0x1589, (short)0x158d, (short)0x1591, (short)0x1594,
			(short)0x1598, (short)0x159c, (short)0x15a0, (short)0x15a4, (short)0x15a7, (short)0x15ab, (short)0x15af, (short)0x15b3, (short)0x15b7, (short)0x15ba, (short)0x15be, (short)0x15c2, (short)0x15c6, (short)0x15c9, (short)0x15cd, (short)0x15d1,
			(short)0x15d5, (short)0x15d8, (short)0x15dc, (short)0x15e0, (short)0x15e4, (short)0x15e8, (short)0x15eb, (short)0x15ef, (short)0x15f3, (short)0x15f7, (short)0x15fa, (short)0x15fe, (short)0x1602, (short)0x1606, (short)0x1609, (short)0x160d,
			(short)0x1611, (short)0x1614, (short)0x1618, (short)0x161c, (short)0x1620, (short)0x1623, (short)0x1627, (short)0x162b, (short)0x162f, (short)0x1632, (short)0x1636, (short)0x163a, (short)0x163e, (short)0x1641, (short)0x1645, (short)0x1649,
			(short)0x164c, (short)0x1650, (short)0x1654, (short)0x1658, (short)0x165b, (short)0x165f, (short)0x1663, (short)0x1666, (short)0x166a, (short)0x166e, (short)0x1671, (short)0x1675, (short)0x1679, (short)0x167d, (short)0x1680, (short)0x1684,
			(short)0x1688, (short)0x168b, (short)0x168f, (short)0x1693, (short)0x1696, (short)0x169a, (short)0x169e, (short)0x16a1, (short)0x16a5, (short)0x16a9, (short)0x16ac, (short)0x16b0, (short)0x16b4, (short)0x16b7, (short)0x16bb, (short)0x16bf,
			// 80
			(short)0x16c2, (short)0x16c6, (short)0x16ca, (short)0x16cd, (short)0x16d1, (short)0x16d5, (short)0x16d8, (short)0x16dc, (short)0x16e0, (short)0x16e3, (short)0x16e7, (short)0x16eb, (short)0x16ee, (short)0x16f2, (short)0x16f6, (short)0x16f9,
			(short)0x16fd, (short)0x1700, (short)0x1704, (short)0x1708, (short)0x170b, (short)0x170f, (short)0x1713, (short)0x1716, (short)0x171a, (short)0x171d, (short)0x1721, (short)0x1725, (short)0x1728, (short)0x172c, (short)0x1730, (short)0x1733,
			(short)0x1737, (short)0x173a, (short)0x173e, (short)0x1742, (short)0x1745, (short)0x1749, (short)0x174c, (short)0x1750, (short)0x1754, (short)0x1757, (short)0x175b, (short)0x175e, (short)0x1762, (short)0x1766, (short)0x1769, (short)0x176d,
			(short)0x1770, (short)0x1774, (short)0x1778, (short)0x177b, (short)0x177f, (short)0x1782, (short)0x1786, (short)0x1789, (short)0x178d, (short)0x1791, (short)0x1794, (short)0x1798, (short)0x179b, (short)0x179f, (short)0x17a2, (short)0x17a6,
			(short)0x17aa, (short)0x17ad, (short)0x17b1, (short)0x17b4, (short)0x17b8, (short)0x17bb, (short)0x17bf, (short)0x17c2, (short)0x17c6, (short)0x17c9, (short)0x17cd, (short)0x17d1, (short)0x17d4, (short)0x17d8, (short)0x17db, (short)0x17df,
			(short)0x17e2, (short)0x17e6, (short)0x17e9, (short)0x17ed, (short)0x17f0, (short)0x17f4, (short)0x17f7, (short)0x17fb, (short)0x17fe, (short)0x1802, (short)0x1806, (short)0x1809, (short)0x180d, (short)0x1810, (short)0x1814, (short)0x1817,
			(short)0x181b, (short)0x181e, (short)0x1822, (short)0x1825, (short)0x1829, (short)0x182c, (short)0x1830, (short)0x1833, (short)0x1837, (short)0x183a, (short)0x183e, (short)0x1841, (short)0x1845, (short)0x1848, (short)0x184c, (short)0x184f,
			(short)0x1853, (short)0x1856, (short)0x185a, (short)0x185d, (short)0x1860, (short)0x1864, (short)0x1867, (short)0x186b, (short)0x186e, (short)0x1872, (short)0x1875, (short)0x1879, (short)0x187c, (short)0x1880, (short)0x1883, (short)0x1887,
			(short)0x188a, (short)0x188e, (short)0x1891, (short)0x1894, (short)0x1898, (short)0x189b, (short)0x189f, (short)0x18a2, (short)0x18a6, (short)0x18a9, (short)0x18ad, (short)0x18b0, (short)0x18b3, (short)0x18b7, (short)0x18ba, (short)0x18be,
			(short)0x18c1, (short)0x18c5, (short)0x18c8, (short)0x18cc, (short)0x18cf, (short)0x18d2, (short)0x18d6, (short)0x18d9, (short)0x18dd, (short)0x18e0, (short)0x18e3, (short)0x18e7, (short)0x18ea, (short)0x18ee, (short)0x18f1, (short)0x18f5,
			(short)0x18f8, (short)0x18fb, (short)0x18ff, (short)0x1902, (short)0x1906, (short)0x1909, (short)0x190c, (short)0x1910, (short)0x1913, (short)0x1917, (short)0x191a, (short)0x191d, (short)0x1921, (short)0x1924, (short)0x1928, (short)0x192b,
			(short)0x192e, (short)0x1932, (short)0x1935, (short)0x1938, (short)0x193c, (short)0x193f, (short)0x1943, (short)0x1946, (short)0x1949, (short)0x194d, (short)0x1950, (short)0x1953, (short)0x1957, (short)0x195a, (short)0x195d, (short)0x1961,
			(short)0x1964, (short)0x1968, (short)0x196b, (short)0x196e, (short)0x1972, (short)0x1975, (short)0x1978, (short)0x197c, (short)0x197f, (short)0x1982, (short)0x1986, (short)0x1989, (short)0x198c, (short)0x1990, (short)0x1993, (short)0x1996,
			(short)0x199a, (short)0x199d, (short)0x19a0, (short)0x19a4, (short)0x19a7, (short)0x19aa, (short)0x19ae, (short)0x19b1, (short)0x19b4, (short)0x19b8, (short)0x19bb, (short)0x19be, (short)0x19c2, (short)0x19c5, (short)0x19c8, (short)0x19cc,
			(short)0x19cf, (short)0x19d2, (short)0x19d5, (short)0x19d9, (short)0x19dc, (short)0x19df, (short)0x19e3, (short)0x19e6, (short)0x19e9, (short)0x19ed, (short)0x19f0, (short)0x19f3, (short)0x19f6, (short)0x19fa, (short)0x19fd, (short)0x1a00,
			(short)0x1a04, (short)0x1a07, (short)0x1a0a, (short)0x1a0d, (short)0x1a11, (short)0x1a14, (short)0x1a17, (short)0x1a1b, (short)0x1a1e, (short)0x1a21, (short)0x1a24, (short)0x1a28, (short)0x1a2b, (short)0x1a2e, (short)0x1a31, (short)0x1a35,
			// 96
			(short)0x1a38, (short)0x1a3b, (short)0x1a3e, (short)0x1a42, (short)0x1a45, (short)0x1a48, (short)0x1a4b, (short)0x1a4f, (short)0x1a52, (short)0x1a55, (short)0x1a58, (short)0x1a5c, (short)0x1a5f, (short)0x1a62, (short)0x1a65, (short)0x1a69,
			(short)0x1a6c, (short)0x1a6f, (short)0x1a72, (short)0x1a76, (short)0x1a79, (short)0x1a7c, (short)0x1a7f, (short)0x1a83, (short)0x1a86, (short)0x1a89, (short)0x1a8c, (short)0x1a8f, (short)0x1a93, (short)0x1a96, (short)0x1a99, (short)0x1a9c,
			(short)0x1a9f, (short)0x1aa3, (short)0x1aa6, (short)0x1aa9, (short)0x1aac, (short)0x1ab0, (short)0x1ab3, (short)0x1ab6, (short)0x1ab9, (short)0x1abc, (short)0x1ac0, (short)0x1ac3, (short)0x1ac6, (short)0x1ac9, (short)0x1acc, (short)0x1acf,
			(short)0x1ad3, (short)0x1ad6, (short)0x1ad9, (short)0x1adc, (short)0x1adf, (short)0x1ae3, (short)0x1ae6, (short)0x1ae9, (short)0x1aec, (short)0x1aef, (short)0x1af2, (short)0x1af6, (short)0x1af9, (short)0x1afc, (short)0x1aff, (short)0x1b02,
			(short)0x1b05, (short)0x1b09, (short)0x1b0c, (short)0x1b0f, (short)0x1b12, (short)0x1b15, (short)0x1b18, (short)0x1b1c, (short)0x1b1f, (short)0x1b22, (short)0x1b25, (short)0x1b28, (short)0x1b2b, (short)0x1b2e, (short)0x1b32, (short)0x1b35,
			(short)0x1b38, (short)0x1b3b, (short)0x1b3e, (short)0x1b41, (short)0x1b44, (short)0x1b48, (short)0x1b4b, (short)0x1b4e, (short)0x1b51, (short)0x1b54, (short)0x1b57, (short)0x1b5a, (short)0x1b5d, (short)0x1b61, (short)0x1b64, (short)0x1b67,
			(short)0x1b6a, (short)0x1b6d, (short)0x1b70, (short)0x1b73, (short)0x1b76, (short)0x1b79, (short)0x1b7d, (short)0x1b80, (short)0x1b83, (short)0x1b86, (short)0x1b89, (short)0x1b8c, (short)0x1b8f, (short)0x1b92, (short)0x1b95, (short)0x1b98,
			(short)0x1b9c, (short)0x1b9f, (short)0x1ba2, (short)0x1ba5, (short)0x1ba8, (short)0x1bab, (short)0x1bae, (short)0x1bb1, (short)0x1bb4, (short)0x1bb7, (short)0x1bba, (short)0x1bbd, (short)0x1bc1, (short)0x1bc4, (short)0x1bc7, (short)0x1bca,
			(short)0x1bcd, (short)0x1bd0, (short)0x1bd3, (short)0x1bd6, (short)0x1bd9, (short)0x1bdc, (short)0x1bdf, (short)0x1be2, (short)0x1be5, (short)0x1be8, (short)0x1beb, (short)0x1bee, (short)0x1bf2, (short)0x1bf5, (short)0x1bf8, (short)0x1bfb,
			(short)0x1bfe, (short)0x1c01, (short)0x1c04, (short)0x1c07, (short)0x1c0a, (short)0x1c0d, (short)0x1c10, (short)0x1c13, (short)0x1c16, (short)0x1c19, (short)0x1c1c, (short)0x1c1f, (short)0x1c22, (short)0x1c25, (short)0x1c28, (short)0x1c2b,
			(short)0x1c2e, (short)0x1c31, (short)0x1c34, (short)0x1c37, (short)0x1c3a, (short)0x1c3d, (short)0x1c40, (short)0x1c43, (short)0x1c46, (short)0x1c49, (short)0x1c4c, (short)0x1c4f, (short)0x1c52, (short)0x1c55, (short)0x1c58, (short)0x1c5b,
			(short)0x1c5e, (short)0x1c61, (short)0x1c64, (short)0x1c67, (short)0x1c6a, (short)0x1c6d, (short)0x1c70, (short)0x1c73, (short)0x1c76, (short)0x1c79, (short)0x1c7c, (short)0x1c7f, (short)0x1c82, (short)0x1c85, (short)0x1c88, (short)0x1c8b,
			(short)0x1c8e, (short)0x1c91, (short)0x1c94, (short)0x1c97, (short)0x1c9a, (short)0x1c9d, (short)0x1ca0, (short)0x1ca3, (short)0x1ca6, (short)0x1ca9, (short)0x1cac, (short)0x1caf, (short)0x1cb2, (short)0x1cb5, (short)0x1cb8, (short)0x1cbb,
			(short)0x1cbe, (short)0x1cc1, (short)0x1cc3, (short)0x1cc6, (short)0x1cc9, (short)0x1ccc, (short)0x1ccf, (short)0x1cd2, (short)0x1cd5, (short)0x1cd8, (short)0x1cdb, (short)0x1cde, (short)0x1ce1, (short)0x1ce4, (short)0x1ce7, (short)0x1cea,
			(short)0x1ced, (short)0x1cf0, (short)0x1cf3, (short)0x1cf5, (short)0x1cf8, (short)0x1cfb, (short)0x1cfe, (short)0x1d01, (short)0x1d04, (short)0x1d07, (short)0x1d0a, (short)0x1d0d, (short)0x1d10, (short)0x1d13, (short)0x1d16, (short)0x1d18,
			(short)0x1d1b, (short)0x1d1e, (short)0x1d21, (short)0x1d24, (short)0x1d27, (short)0x1d2a, (short)0x1d2d, (short)0x1d30, (short)0x1d33, (short)0x1d35, (short)0x1d38, (short)0x1d3b, (short)0x1d3e, (short)0x1d41, (short)0x1d44, (short)0x1d47,
			// 112
			(short)0x1d4a, (short)0x1d4d, (short)0x1d4f, (short)0x1d52, (short)0x1d55, (short)0x1d58, (short)0x1d5b, (short)0x1d5e, (short)0x1d61, (short)0x1d64, (short)0x1d66, (short)0x1d69, (short)0x1d6c, (short)0x1d6f, (short)0x1d72, (short)0x1d75,
			(short)0x1d78, (short)0x1d7b, (short)0x1d7d, (short)0x1d80, (short)0x1d83, (short)0x1d86, (short)0x1d89, (short)0x1d8c, (short)0x1d8e, (short)0x1d91, (short)0x1d94, (short)0x1d97, (short)0x1d9a, (short)0x1d9d, (short)0x1da0, (short)0x1da2,
			(short)0x1da5, (short)0x1da8, (short)0x1dab, (short)0x1dae, (short)0x1db1, (short)0x1db3, (short)0x1db6, (short)0x1db9, (short)0x1dbc, (short)0x1dbf, (short)0x1dc2, (short)0x1dc4, (short)0x1dc7, (short)0x1dca, (short)0x1dcd, (short)0x1dd0,
			(short)0x1dd3, (short)0x1dd5, (short)0x1dd8, (short)0x1ddb, (short)0x1dde, (short)0x1de1, (short)0x1de3, (short)0x1de6, (short)0x1de9, (short)0x1dec, (short)0x1def, (short)0x1df1, (short)0x1df4, (short)0x1df7, (short)0x1dfa, (short)0x1dfd,
			(short)0x1dff, (short)0x1e02, (short)0x1e05, (short)0x1e08, (short)0x1e0b, (short)0x1e0d, (short)0x1e10, (short)0x1e13, (short)0x1e16, (short)0x1e19, (short)0x1e1b, (short)0x1e1e, (short)0x1e21, (short)0x1e24, (short)0x1e26, (short)0x1e29,
			(short)0x1e2c, (short)0x1e2f, (short)0x1e32, (short)0x1e34, (short)0x1e37, (short)0x1e3a, (short)0x1e3d, (short)0x1e3f, (short)0x1e42, (short)0x1e45, (short)0x1e48, (short)0x1e4a, (short)0x1e4d, (short)0x1e50, (short)0x1e53, (short)0x1e55,
			(short)0x1e58, (short)0x1e5b, (short)0x1e5e, (short)0x1e60, (short)0x1e63, (short)0x1e66, (short)0x1e69, (short)0x1e6b, (short)0x1e6e, (short)0x1e71, (short)0x1e74, (short)0x1e76, (short)0x1e79, (short)0x1e7c, (short)0x1e7f, (short)0x1e81,
			(short)0x1e84, (short)0x1e87, (short)0x1e8a, (short)0x1e8c, (short)0x1e8f, (short)0x1e92, (short)0x1e94, (short)0x1e97, (short)0x1e9a, (short)0x1e9d, (short)0x1e9f, (short)0x1ea2, (short)0x1ea5, (short)0x1ea8, (short)0x1eaa, (short)0x1ead,
			(short)0x1eb0, (short)0x1eb2, (short)0x1eb5, (short)0x1eb8, (short)0x1eba, (short)0x1ebd, (short)0x1ec0, (short)0x1ec3, (short)0x1ec5, (short)0x1ec8, (short)0x1ecb, (short)0x1ecd, (short)0x1ed0, (short)0x1ed3, (short)0x1ed5, (short)0x1ed8,
			(short)0x1edb, (short)0x1ede, (short)0x1ee0, (short)0x1ee3, (short)0x1ee6, (short)0x1ee8, (short)0x1eeb, (short)0x1eee, (short)0x1ef0, (short)0x1ef3, (short)0x1ef6, (short)0x1ef8, (short)0x1efb, (short)0x1efe, (short)0x1f00, (short)0x1f03,
			(short)0x1f06, (short)0x1f08, (short)0x1f0b, (short)0x1f0e, (short)0x1f10, (short)0x1f13, (short)0x1f16, (short)0x1f18, (short)0x1f1b, (short)0x1f1e, (short)0x1f20, (short)0x1f23, (short)0x1f26, (short)0x1f28, (short)0x1f2b, (short)0x1f2e,
			(short)0x1f30, (short)0x1f33, (short)0x1f36, (short)0x1f38, (short)0x1f3b, (short)0x1f3d, (short)0x1f40, (short)0x1f43, (short)0x1f45, (short)0x1f48, (short)0x1f4b, (short)0x1f4d, (short)0x1f50, (short)0x1f53, (short)0x1f55, (short)0x1f58,
			(short)0x1f5a, (short)0x1f5d, (short)0x1f60, (short)0x1f62, (short)0x1f65, (short)0x1f68, (short)0x1f6a, (short)0x1f6d, (short)0x1f6f, (short)0x1f72, (short)0x1f75, (short)0x1f77, (short)0x1f7a, (short)0x1f7c, (short)0x1f7f, (short)0x1f82,
			(short)0x1f84, (short)0x1f87, (short)0x1f8a, (short)0x1f8c, (short)0x1f8f, (short)0x1f91, (short)0x1f94, (short)0x1f97, (short)0x1f99, (short)0x1f9c, (short)0x1f9e, (short)0x1fa1, (short)0x1fa4, (short)0x1fa6, (short)0x1fa9, (short)0x1fab,
			(short)0x1fae, (short)0x1fb0, (short)0x1fb3, (short)0x1fb6, (short)0x1fb8, (short)0x1fbb, (short)0x1fbd, (short)0x1fc0, (short)0x1fc3, (short)0x1fc5, (short)0x1fc8, (short)0x1fca, (short)0x1fcd, (short)0x1fcf, (short)0x1fd2, (short)0x1fd5,
			(short)0x1fd7, (short)0x1fda, (short)0x1fdc, (short)0x1fdf, (short)0x1fe1, (short)0x1fe4, (short)0x1fe6, (short)0x1fe9, (short)0x1fec, (short)0x1fee, (short)0x1ff1, (short)0x1ff3, (short)0x1ff6, (short)0x1ff8, (short)0x1ffb, (short)0x1ffd };
	// 128
	/** unsigned int16 */
	static short[] w_perspective_table_2 = {
			// 0
			(short)0x0, (short)0x10, (short)0x20, (short)0x30, (short)0x40, (short)0x50, (short)0x60, (short)0x70, (short)0x80, (short)0x90, (short)0xa0, (short)0xb0, (short)0xc0, (short)0xd0, (short)0xe0, (short)0xf0,
			(short)0x100, (short)0x110, (short)0x120, (short)0x130, (short)0x140, (short)0x150, (short)0x160, (short)0x170, (short)0x180, (short)0x190, (short)0x1a0, (short)0x1b0, (short)0x1c0, (short)0x1d0, (short)0x1e0, (short)0x1f0,
			(short)0x200, (short)0x210, (short)0x220, (short)0x230, (short)0x240, (short)0x250, (short)0x260, (short)0x270, (short)0x280, (short)0x290, (short)0x2a0, (short)0x2b0, (short)0x2c0, (short)0x2d0, (short)0x2e0, (short)0x2f0,
			(short)0x300, (short)0x310, (short)0x320, (short)0x330, (short)0x340, (short)0x350, (short)0x360, (short)0x370, (short)0x380, (short)0x390, (short)0x3a0, (short)0x3b0, (short)0x3c0, (short)0x3d0, (short)0x3e0, (short)0x3f0,
			(short)0x400, (short)0x410, (short)0x420, (short)0x430, (short)0x440, (short)0x450, (short)0x460, (short)0x470, (short)0x480, (short)0x490, (short)0x4a0, (short)0x4b0, (short)0x4c0, (short)0x4d0, (short)0x4e0, (short)0x4f0,
			(short)0x500, (short)0x50f, (short)0x51f, (short)0x52f, (short)0x53f, (short)0x54f, (short)0x55f, (short)0x56f, (short)0x57f, (short)0x58f, (short)0x59f, (short)0x5af, (short)0x5bf, (short)0x5cf, (short)0x5df, (short)0x5ef,
			(short)0x5ff, (short)0x60f, (short)0x61f, (short)0x62f, (short)0x63f, (short)0x64f, (short)0x65f, (short)0x66f, (short)0x67f, (short)0x68f, (short)0x69f, (short)0x6af, (short)0x6bf, (short)0x6cf, (short)0x6df, (short)0x6ef,
			(short)0x6ff, (short)0x70f, (short)0x71f, (short)0x72f, (short)0x73f, (short)0x74e, (short)0x75e, (short)0x76e, (short)0x77e, (short)0x78e, (short)0x79e, (short)0x7ae, (short)0x7be, (short)0x7ce, (short)0x7de, (short)0x7ee,
			(short)0x7fe, (short)0x80e, (short)0x81e, (short)0x82e, (short)0x83e, (short)0x84e, (short)0x85e, (short)0x86e, (short)0x87e, (short)0x88e, (short)0x89d, (short)0x8ad, (short)0x8bd, (short)0x8cd, (short)0x8dd, (short)0x8ed,
			(short)0x8fd, (short)0x90d, (short)0x91d, (short)0x92d, (short)0x93d, (short)0x94d, (short)0x95d, (short)0x96d, (short)0x97d, (short)0x98d, (short)0x99d, (short)0x9ac, (short)0x9bc, (short)0x9cc, (short)0x9dc, (short)0x9ec,
			(short)0x9fc, (short)0xa0c, (short)0xa1c, (short)0xa2c, (short)0xa3c, (short)0xa4c, (short)0xa5c, (short)0xa6c, (short)0xa7b, (short)0xa8b, (short)0xa9b, (short)0xaab, (short)0xabb, (short)0xacb, (short)0xadb, (short)0xaeb,
			(short)0xafb, (short)0xb0b, (short)0xb1b, (short)0xb2b, (short)0xb3a, (short)0xb4a, (short)0xb5a, (short)0xb6a, (short)0xb7a, (short)0xb8a, (short)0xb9a, (short)0xbaa, (short)0xbba, (short)0xbca, (short)0xbd9, (short)0xbe9,
			(short)0xbf9, (short)0xc09, (short)0xc19, (short)0xc29, (short)0xc39, (short)0xc49, (short)0xc59, (short)0xc69, (short)0xc78, (short)0xc88, (short)0xc98, (short)0xca8, (short)0xcb8, (short)0xcc8, (short)0xcd8, (short)0xce8,
			(short)0xcf7, (short)0xd07, (short)0xd17, (short)0xd27, (short)0xd37, (short)0xd47, (short)0xd57, (short)0xd67, (short)0xd76, (short)0xd86, (short)0xd96, (short)0xda6, (short)0xdb6, (short)0xdc6, (short)0xdd6, (short)0xde5,
			(short)0xdf5, (short)0xe05, (short)0xe15, (short)0xe25, (short)0xe35, (short)0xe45, (short)0xe54, (short)0xe64, (short)0xe74, (short)0xe84, (short)0xe94, (short)0xea4, (short)0xeb4, (short)0xec3, (short)0xed3, (short)0xee3,
			(short)0xef3, (short)0xf03, (short)0xf13, (short)0xf22, (short)0xf32, (short)0xf42, (short)0xf52, (short)0xf62, (short)0xf72, (short)0xf81, (short)0xf91, (short)0xfa1, (short)0xfb1, (short)0xfc1, (short)0xfd0, (short)0xfe0,
			// 16
			(short)0xff0, (short)0x1000, (short)0x1010, (short)0x1020, (short)0x102f, (short)0x103f, (short)0x104f, (short)0x105f, (short)0x106f, (short)0x107e, (short)0x108e, (short)0x109e, (short)0x10ae, (short)0x10be, (short)0x10cd, (short)0x10dd,
			(short)0x10ed, (short)0x10fd, (short)0x110d, (short)0x111c, (short)0x112c, (short)0x113c, (short)0x114c, (short)0x115b, (short)0x116b, (short)0x117b, (short)0x118b, (short)0x119b, (short)0x11aa, (short)0x11ba, (short)0x11ca, (short)0x11da,
			(short)0x11e9, (short)0x11f9, (short)0x1209, (short)0x1219, (short)0x1228, (short)0x1238, (short)0x1248, (short)0x1258, (short)0x1268, (short)0x1277, (short)0x1287, (short)0x1297, (short)0x12a7, (short)0x12b6, (short)0x12c6, (short)0x12d6,
			(short)0x12e5, (short)0x12f5, (short)0x1305, (short)0x1315, (short)0x1324, (short)0x1334, (short)0x1344, (short)0x1354, (short)0x1363, (short)0x1373, (short)0x1383, (short)0x1393, (short)0x13a2, (short)0x13b2, (short)0x13c2, (short)0x13d1,
			(short)0x13e1, (short)0x13f1, (short)0x1401, (short)0x1410, (short)0x1420, (short)0x1430, (short)0x143f, (short)0x144f, (short)0x145f, (short)0x146e, (short)0x147e, (short)0x148e, (short)0x149e, (short)0x14ad, (short)0x14bd, (short)0x14cd,
			(short)0x14dc, (short)0x14ec, (short)0x14fc, (short)0x150b, (short)0x151b, (short)0x152b, (short)0x153a, (short)0x154a, (short)0x155a, (short)0x1569, (short)0x1579, (short)0x1589, (short)0x1598, (short)0x15a8, (short)0x15b8, (short)0x15c7,
			(short)0x15d7, (short)0x15e7, (short)0x15f6, (short)0x1606, (short)0x1616, (short)0x1625, (short)0x1635, (short)0x1645, (short)0x1654, (short)0x1664, (short)0x1673, (short)0x1683, (short)0x1693, (short)0x16a2, (short)0x16b2, (short)0x16c2,
			(short)0x16d1, (short)0x16e1, (short)0x16f0, (short)0x1700, (short)0x1710, (short)0x171f, (short)0x172f, (short)0x173f, (short)0x174e, (short)0x175e, (short)0x176d, (short)0x177d, (short)0x178d, (short)0x179c, (short)0x17ac, (short)0x17bb,
			(short)0x17cb, (short)0x17db, (short)0x17ea, (short)0x17fa, (short)0x1809, (short)0x1819, (short)0x1828, (short)0x1838, (short)0x1848, (short)0x1857, (short)0x1867, (short)0x1876, (short)0x1886, (short)0x1895, (short)0x18a5, (short)0x18b5,
			(short)0x18c4, (short)0x18d4, (short)0x18e3, (short)0x18f3, (short)0x1902, (short)0x1912, (short)0x1921, (short)0x1931, (short)0x1940, (short)0x1950, (short)0x1960, (short)0x196f, (short)0x197f, (short)0x198e, (short)0x199e, (short)0x19ad,
			(short)0x19bd, (short)0x19cc, (short)0x19dc, (short)0x19eb, (short)0x19fb, (short)0x1a0a, (short)0x1a1a, (short)0x1a29, (short)0x1a39, (short)0x1a48, (short)0x1a58, (short)0x1a67, (short)0x1a77, (short)0x1a86, (short)0x1a96, (short)0x1aa5,
			(short)0x1ab5, (short)0x1ac4, (short)0x1ad4, (short)0x1ae3, (short)0x1af3, (short)0x1b02, (short)0x1b12, (short)0x1b21, (short)0x1b31, (short)0x1b40, (short)0x1b50, (short)0x1b5f, (short)0x1b6e, (short)0x1b7e, (short)0x1b8d, (short)0x1b9d,
			(short)0x1bac, (short)0x1bbc, (short)0x1bcb, (short)0x1bdb, (short)0x1bea, (short)0x1bf9, (short)0x1c09, (short)0x1c18, (short)0x1c28, (short)0x1c37, (short)0x1c47, (short)0x1c56, (short)0x1c65, (short)0x1c75, (short)0x1c84, (short)0x1c94,
			(short)0x1ca3, (short)0x1cb3, (short)0x1cc2, (short)0x1cd1, (short)0x1ce1, (short)0x1cf0, (short)0x1d00, (short)0x1d0f, (short)0x1d1e, (short)0x1d2e, (short)0x1d3d, (short)0x1d4c, (short)0x1d5c, (short)0x1d6b, (short)0x1d7b, (short)0x1d8a,
			(short)0x1d99, (short)0x1da9, (short)0x1db8, (short)0x1dc7, (short)0x1dd7, (short)0x1de6, (short)0x1df6, (short)0x1e05, (short)0x1e14, (short)0x1e24, (short)0x1e33, (short)0x1e42, (short)0x1e52, (short)0x1e61, (short)0x1e70, (short)0x1e80,
			(short)0x1e8f, (short)0x1e9e, (short)0x1eae, (short)0x1ebd, (short)0x1ecc, (short)0x1edc, (short)0x1eeb, (short)0x1efa, (short)0x1f09, (short)0x1f19, (short)0x1f28, (short)0x1f37, (short)0x1f47, (short)0x1f56, (short)0x1f65, (short)0x1f75,
			// 32
			(short)0x1f84, (short)0x1f93, (short)0x1fa2, (short)0x1fb2, (short)0x1fc1, (short)0x1fd0, (short)0x1fe0, (short)0x1fef, (short)0x1ffe, (short)0x200d, (short)0x201d, (short)0x202c, (short)0x203b, (short)0x204a, (short)0x205a, (short)0x2069,
			(short)0x2078, (short)0x2087, (short)0x2097, (short)0x20a6, (short)0x20b5, (short)0x20c4, (short)0x20d4, (short)0x20e3, (short)0x20f2, (short)0x2101, (short)0x2110, (short)0x2120, (short)0x212f, (short)0x213e, (short)0x214d, (short)0x215c,
			(short)0x216c, (short)0x217b, (short)0x218a, (short)0x2199, (short)0x21a8, (short)0x21b8, (short)0x21c7, (short)0x21d6, (short)0x21e5, (short)0x21f4, (short)0x2204, (short)0x2213, (short)0x2222, (short)0x2231, (short)0x2240, (short)0x224f,
			(short)0x225f, (short)0x226e, (short)0x227d, (short)0x228c, (short)0x229b, (short)0x22aa, (short)0x22b9, (short)0x22c9, (short)0x22d8, (short)0x22e7, (short)0x22f6, (short)0x2305, (short)0x2314, (short)0x2323, (short)0x2332, (short)0x2342,
			(short)0x2351, (short)0x2360, (short)0x236f, (short)0x237e, (short)0x238d, (short)0x239c, (short)0x23ab, (short)0x23ba, (short)0x23c9, (short)0x23d8, (short)0x23e8, (short)0x23f7, (short)0x2406, (short)0x2415, (short)0x2424, (short)0x2433,
			(short)0x2442, (short)0x2451, (short)0x2460, (short)0x246f, (short)0x247e, (short)0x248d, (short)0x249c, (short)0x24ab, (short)0x24ba, (short)0x24c9, (short)0x24d8, (short)0x24e8, (short)0x24f7, (short)0x2506, (short)0x2515, (short)0x2524,
			(short)0x2533, (short)0x2542, (short)0x2551, (short)0x2560, (short)0x256f, (short)0x257e, (short)0x258d, (short)0x259c, (short)0x25ab, (short)0x25ba, (short)0x25c9, (short)0x25d8, (short)0x25e7, (short)0x25f6, (short)0x2605, (short)0x2613,
			(short)0x2622, (short)0x2631, (short)0x2640, (short)0x264f, (short)0x265e, (short)0x266d, (short)0x267c, (short)0x268b, (short)0x269a, (short)0x26a9, (short)0x26b8, (short)0x26c7, (short)0x26d6, (short)0x26e5, (short)0x26f4, (short)0x2703,
			(short)0x2712, (short)0x2720, (short)0x272f, (short)0x273e, (short)0x274d, (short)0x275c, (short)0x276b, (short)0x277a, (short)0x2789, (short)0x2798, (short)0x27a7, (short)0x27b5, (short)0x27c4, (short)0x27d3, (short)0x27e2, (short)0x27f1,
			(short)0x2800, (short)0x280f, (short)0x281d, (short)0x282c, (short)0x283b, (short)0x284a, (short)0x2859, (short)0x2868, (short)0x2877, (short)0x2885, (short)0x2894, (short)0x28a3, (short)0x28b2, (short)0x28c1, (short)0x28d0, (short)0x28de,
			(short)0x28ed, (short)0x28fc, (short)0x290b, (short)0x291a, (short)0x2928, (short)0x2937, (short)0x2946, (short)0x2955, (short)0x2964, (short)0x2972, (short)0x2981, (short)0x2990, (short)0x299f, (short)0x29ae, (short)0x29bc, (short)0x29cb,
			(short)0x29da, (short)0x29e9, (short)0x29f7, (short)0x2a06, (short)0x2a15, (short)0x2a24, (short)0x2a32, (short)0x2a41, (short)0x2a50, (short)0x2a5f, (short)0x2a6d, (short)0x2a7c, (short)0x2a8b, (short)0x2a99, (short)0x2aa8, (short)0x2ab7,
			(short)0x2ac6, (short)0x2ad4, (short)0x2ae3, (short)0x2af2, (short)0x2b00, (short)0x2b0f, (short)0x2b1e, (short)0x2b2c, (short)0x2b3b, (short)0x2b4a, (short)0x2b58, (short)0x2b67, (short)0x2b76, (short)0x2b84, (short)0x2b93, (short)0x2ba2,
			(short)0x2bb0, (short)0x2bbf, (short)0x2bce, (short)0x2bdc, (short)0x2beb, (short)0x2bfa, (short)0x2c08, (short)0x2c17, (short)0x2c26, (short)0x2c34, (short)0x2c43, (short)0x2c51, (short)0x2c60, (short)0x2c6f, (short)0x2c7d, (short)0x2c8c,
			(short)0x2c9b, (short)0x2ca9, (short)0x2cb8, (short)0x2cc6, (short)0x2cd5, (short)0x2ce3, (short)0x2cf2, (short)0x2d01, (short)0x2d0f, (short)0x2d1e, (short)0x2d2c, (short)0x2d3b, (short)0x2d49, (short)0x2d58, (short)0x2d67, (short)0x2d75,
			(short)0x2d84, (short)0x2d92, (short)0x2da1, (short)0x2daf, (short)0x2dbe, (short)0x2dcc, (short)0x2ddb, (short)0x2de9, (short)0x2df8, (short)0x2e06, (short)0x2e15, (short)0x2e23, (short)0x2e32, (short)0x2e40, (short)0x2e4f, (short)0x2e5d,
			// 48
			(short)0x2e6c, (short)0x2e7a, (short)0x2e89, (short)0x2e97, (short)0x2ea6, (short)0x2eb4, (short)0x2ec3, (short)0x2ed1, (short)0x2ee0, (short)0x2eee, (short)0x2efd, (short)0x2f0b, (short)0x2f1a, (short)0x2f28, (short)0x2f36, (short)0x2f45,
			(short)0x2f53, (short)0x2f62, (short)0x2f70, (short)0x2f7f, (short)0x2f8d, (short)0x2f9b, (short)0x2faa, (short)0x2fb8, (short)0x2fc7, (short)0x2fd5, (short)0x2fe3, (short)0x2ff2, (short)0x3000, (short)0x300f, (short)0x301d, (short)0x302b,
			(short)0x303a, (short)0x3048, (short)0x3057, (short)0x3065, (short)0x3073, (short)0x3082, (short)0x3090, (short)0x309e, (short)0x30ad, (short)0x30bb, (short)0x30c9, (short)0x30d8, (short)0x30e6, (short)0x30f4, (short)0x3103, (short)0x3111,
			(short)0x311f, (short)0x312e, (short)0x313c, (short)0x314a, (short)0x3159, (short)0x3167, (short)0x3175, (short)0x3183, (short)0x3192, (short)0x31a0, (short)0x31ae, (short)0x31bd, (short)0x31cb, (short)0x31d9, (short)0x31e7, (short)0x31f6,
			(short)0x3204, (short)0x3212, (short)0x3220, (short)0x322f, (short)0x323d, (short)0x324b, (short)0x3259, (short)0x3268, (short)0x3276, (short)0x3284, (short)0x3292, (short)0x32a0, (short)0x32af, (short)0x32bd, (short)0x32cb, (short)0x32d9,
			(short)0x32e8, (short)0x32f6, (short)0x3304, (short)0x3312, (short)0x3320, (short)0x332e, (short)0x333d, (short)0x334b, (short)0x3359, (short)0x3367, (short)0x3375, (short)0x3383, (short)0x3392, (short)0x33a0, (short)0x33ae, (short)0x33bc,
			(short)0x33ca, (short)0x33d8, (short)0x33e6, (short)0x33f5, (short)0x3403, (short)0x3411, (short)0x341f, (short)0x342d, (short)0x343b, (short)0x3449, (short)0x3457, (short)0x3466, (short)0x3474, (short)0x3482, (short)0x3490, (short)0x349e,
			(short)0x34ac, (short)0x34ba, (short)0x34c8, (short)0x34d6, (short)0x34e4, (short)0x34f2, (short)0x3500, (short)0x350e, (short)0x351c, (short)0x352b, (short)0x3539, (short)0x3547, (short)0x3555, (short)0x3563, (short)0x3571, (short)0x357f,
			(short)0x358d, (short)0x359b, (short)0x35a9, (short)0x35b7, (short)0x35c5, (short)0x35d3, (short)0x35e1, (short)0x35ef, (short)0x35fd, (short)0x360b, (short)0x3619, (short)0x3627, (short)0x3635, (short)0x3643, (short)0x3651, (short)0x365f,
			(short)0x366d, (short)0x367a, (short)0x3688, (short)0x3696, (short)0x36a4, (short)0x36b2, (short)0x36c0, (short)0x36ce, (short)0x36dc, (short)0x36ea, (short)0x36f8, (short)0x3706, (short)0x3714, (short)0x3722, (short)0x3730, (short)0x373d,
			(short)0x374b, (short)0x3759, (short)0x3767, (short)0x3775, (short)0x3783, (short)0x3791, (short)0x379f, (short)0x37ac, (short)0x37ba, (short)0x37c8, (short)0x37d6, (short)0x37e4, (short)0x37f2, (short)0x3800, (short)0x380d, (short)0x381b,
			(short)0x3829, (short)0x3837, (short)0x3845, (short)0x3853, (short)0x3860, (short)0x386e, (short)0x387c, (short)0x388a, (short)0x3898, (short)0x38a5, (short)0x38b3, (short)0x38c1, (short)0x38cf, (short)0x38dd, (short)0x38ea, (short)0x38f8,
			(short)0x3906, (short)0x3914, (short)0x3921, (short)0x392f, (short)0x393d, (short)0x394b, (short)0x3958, (short)0x3966, (short)0x3974, (short)0x3982, (short)0x398f, (short)0x399d, (short)0x39ab, (short)0x39b9, (short)0x39c6, (short)0x39d4,
			(short)0x39e2, (short)0x39ef, (short)0x39fd, (short)0x3a0b, (short)0x3a18, (short)0x3a26, (short)0x3a34, (short)0x3a42, (short)0x3a4f, (short)0x3a5d, (short)0x3a6b, (short)0x3a78, (short)0x3a86, (short)0x3a94, (short)0x3aa1, (short)0x3aaf,
			(short)0x3abc, (short)0x3aca, (short)0x3ad8, (short)0x3ae5, (short)0x3af3, (short)0x3b01, (short)0x3b0e, (short)0x3b1c, (short)0x3b29, (short)0x3b37, (short)0x3b45, (short)0x3b52, (short)0x3b60, (short)0x3b6d, (short)0x3b7b, (short)0x3b89,
			(short)0x3b96, (short)0x3ba4, (short)0x3bb1, (short)0x3bbf, (short)0x3bcd, (short)0x3bda, (short)0x3be8, (short)0x3bf5, (short)0x3c03, (short)0x3c10, (short)0x3c1e, (short)0x3c2b, (short)0x3c39, (short)0x3c46, (short)0x3c54, (short)0x3c61,
			// 64
			(short)0x3c6f, (short)0x3c7c, (short)0x3c8a, (short)0x3c97, (short)0x3ca5, (short)0x3cb2, (short)0x3cc0, (short)0x3ccd, (short)0x3cdb, (short)0x3ce8, (short)0x3cf6, (short)0x3d03, (short)0x3d11, (short)0x3d1e, (short)0x3d2c, (short)0x3d39,
			(short)0x3d47, (short)0x3d54, (short)0x3d62, (short)0x3d6f, (short)0x3d7c, (short)0x3d8a, (short)0x3d97, (short)0x3da5, (short)0x3db2, (short)0x3dc0, (short)0x3dcd, (short)0x3dda, (short)0x3de8, (short)0x3df5, (short)0x3e03, (short)0x3e10,
			(short)0x3e1d, (short)0x3e2b, (short)0x3e38, (short)0x3e45, (short)0x3e53, (short)0x3e60, (short)0x3e6e, (short)0x3e7b, (short)0x3e88, (short)0x3e96, (short)0x3ea3, (short)0x3eb0, (short)0x3ebe, (short)0x3ecb, (short)0x3ed8, (short)0x3ee6,
			(short)0x3ef3, (short)0x3f00, (short)0x3f0e, (short)0x3f1b, (short)0x3f28, (short)0x3f35, (short)0x3f43, (short)0x3f50, (short)0x3f5d, (short)0x3f6b, (short)0x3f78, (short)0x3f85, (short)0x3f92, (short)0x3fa0, (short)0x3fad, (short)0x3fba,
			(short)0x3fc7, (short)0x3fd5, (short)0x3fe2, (short)0x3fef, (short)0x3ffc, (short)0x400a, (short)0x4017, (short)0x4024, (short)0x4031, (short)0x403f, (short)0x404c, (short)0x4059, (short)0x4066, (short)0x4073, (short)0x4081, (short)0x408e,
			(short)0x409b, (short)0x40a8, (short)0x40b5, (short)0x40c3, (short)0x40d0, (short)0x40dd, (short)0x40ea, (short)0x40f7, (short)0x4104, (short)0x4112, (short)0x411f, (short)0x412c, (short)0x4139, (short)0x4146, (short)0x4153, (short)0x4160,
			(short)0x416d, (short)0x417b, (short)0x4188, (short)0x4195, (short)0x41a2, (short)0x41af, (short)0x41bc, (short)0x41c9, (short)0x41d6, (short)0x41e3, (short)0x41f0, (short)0x41fe, (short)0x420b, (short)0x4218, (short)0x4225, (short)0x4232,
			(short)0x423f, (short)0x424c, (short)0x4259, (short)0x4266, (short)0x4273, (short)0x4280, (short)0x428d, (short)0x429a, (short)0x42a7, (short)0x42b4, (short)0x42c1, (short)0x42ce, (short)0x42db, (short)0x42e8, (short)0x42f5, (short)0x4302,
			(short)0x430f, (short)0x431c, (short)0x4329, (short)0x4336, (short)0x4343, (short)0x4350, (short)0x435d, (short)0x436a, (short)0x4377, (short)0x4384, (short)0x4391, (short)0x439e, (short)0x43ab, (short)0x43b8, (short)0x43c5, (short)0x43d2,
			(short)0x43df, (short)0x43eb, (short)0x43f8, (short)0x4405, (short)0x4412, (short)0x441f, (short)0x442c, (short)0x4439, (short)0x4446, (short)0x4453, (short)0x4460, (short)0x446c, (short)0x4479, (short)0x4486, (short)0x4493, (short)0x44a0,
			(short)0x44ad, (short)0x44ba, (short)0x44c6, (short)0x44d3, (short)0x44e0, (short)0x44ed, (short)0x44fa, (short)0x4507, (short)0x4513, (short)0x4520, (short)0x452d, (short)0x453a, (short)0x4547, (short)0x4553, (short)0x4560, (short)0x456d,
			(short)0x457a, (short)0x4587, (short)0x4593, (short)0x45a0, (short)0x45ad, (short)0x45ba, (short)0x45c7, (short)0x45d3, (short)0x45e0, (short)0x45ed, (short)0x45fa, (short)0x4606, (short)0x4613, (short)0x4620, (short)0x462c, (short)0x4639,
			(short)0x4646, (short)0x4653, (short)0x465f, (short)0x466c, (short)0x4679, (short)0x4685, (short)0x4692, (short)0x469f, (short)0x46ac, (short)0x46b8, (short)0x46c5, (short)0x46d2, (short)0x46de, (short)0x46eb, (short)0x46f8, (short)0x4704,
			(short)0x4711, (short)0x471e, (short)0x472a, (short)0x4737, (short)0x4744, (short)0x4750, (short)0x475d, (short)0x4769, (short)0x4776, (short)0x4783, (short)0x478f, (short)0x479c, (short)0x47a8, (short)0x47b5, (short)0x47c2, (short)0x47ce,
			(short)0x47db, (short)0x47e7, (short)0x47f4, (short)0x4801, (short)0x480d, (short)0x481a, (short)0x4826, (short)0x4833, (short)0x483f, (short)0x484c, (short)0x4859, (short)0x4865, (short)0x4872, (short)0x487e, (short)0x488b, (short)0x4897,
			(short)0x48a4, (short)0x48b0, (short)0x48bd, (short)0x48c9, (short)0x48d6, (short)0x48e2, (short)0x48ef, (short)0x48fb, (short)0x4908, (short)0x4914, (short)0x4921, (short)0x492d, (short)0x493a, (short)0x4946, (short)0x4953, (short)0x495f,
			// 80
			(short)0x496b, (short)0x4978, (short)0x4984, (short)0x4991, (short)0x499d, (short)0x49aa, (short)0x49b6, (short)0x49c3, (short)0x49cf, (short)0x49db, (short)0x49e8, (short)0x49f4, (short)0x4a01, (short)0x4a0d, (short)0x4a19, (short)0x4a26,
			(short)0x4a32, (short)0x4a3f, (short)0x4a4b, (short)0x4a57, (short)0x4a64, (short)0x4a70, (short)0x4a7c, (short)0x4a89, (short)0x4a95, (short)0x4aa1, (short)0x4aae, (short)0x4aba, (short)0x4ac6, (short)0x4ad3, (short)0x4adf, (short)0x4aeb,
			(short)0x4af8, (short)0x4b04, (short)0x4b10, (short)0x4b1d, (short)0x4b29, (short)0x4b35, (short)0x4b42, (short)0x4b4e, (short)0x4b5a, (short)0x4b66, (short)0x4b73, (short)0x4b7f, (short)0x4b8b, (short)0x4b98, (short)0x4ba4, (short)0x4bb0,
			(short)0x4bbc, (short)0x4bc9, (short)0x4bd5, (short)0x4be1, (short)0x4bed, (short)0x4bfa, (short)0x4c06, (short)0x4c12, (short)0x4c1e, (short)0x4c2a, (short)0x4c37, (short)0x4c43, (short)0x4c4f, (short)0x4c5b, (short)0x4c67, (short)0x4c74,
			(short)0x4c80, (short)0x4c8c, (short)0x4c98, (short)0x4ca4, (short)0x4cb0, (short)0x4cbd, (short)0x4cc9, (short)0x4cd5, (short)0x4ce1, (short)0x4ced, (short)0x4cf9, (short)0x4d06, (short)0x4d12, (short)0x4d1e, (short)0x4d2a, (short)0x4d36,
			(short)0x4d42, (short)0x4d4e, (short)0x4d5a, (short)0x4d66, (short)0x4d73, (short)0x4d7f, (short)0x4d8b, (short)0x4d97, (short)0x4da3, (short)0x4daf, (short)0x4dbb, (short)0x4dc7, (short)0x4dd3, (short)0x4ddf, (short)0x4deb, (short)0x4df7,
			(short)0x4e03, (short)0x4e0f, (short)0x4e1c, (short)0x4e28, (short)0x4e34, (short)0x4e40, (short)0x4e4c, (short)0x4e58, (short)0x4e64, (short)0x4e70, (short)0x4e7c, (short)0x4e88, (short)0x4e94, (short)0x4ea0, (short)0x4eac, (short)0x4eb8,
			(short)0x4ec4, (short)0x4ed0, (short)0x4edc, (short)0x4ee8, (short)0x4ef4, (short)0x4f00, (short)0x4f0b, (short)0x4f17, (short)0x4f23, (short)0x4f2f, (short)0x4f3b, (short)0x4f47, (short)0x4f53, (short)0x4f5f, (short)0x4f6b, (short)0x4f77,
			(short)0x4f83, (short)0x4f8f, (short)0x4f9b, (short)0x4fa7, (short)0x4fb2, (short)0x4fbe, (short)0x4fca, (short)0x4fd6, (short)0x4fe2, (short)0x4fee, (short)0x4ffa, (short)0x5006, (short)0x5011, (short)0x501d, (short)0x5029, (short)0x5035,
			(short)0x5041, (short)0x504d, (short)0x5059, (short)0x5064, (short)0x5070, (short)0x507c, (short)0x5088, (short)0x5094, (short)0x50a0, (short)0x50ab, (short)0x50b7, (short)0x50c3, (short)0x50cf, (short)0x50db, (short)0x50e6, (short)0x50f2,
			(short)0x50fe, (short)0x510a, (short)0x5115, (short)0x5121, (short)0x512d, (short)0x5139, (short)0x5144, (short)0x5150, (short)0x515c, (short)0x5168, (short)0x5173, (short)0x517f, (short)0x518b, (short)0x5197, (short)0x51a2, (short)0x51ae,
			(short)0x51ba, (short)0x51c5, (short)0x51d1, (short)0x51dd, (short)0x51e9, (short)0x51f4, (short)0x5200, (short)0x520c, (short)0x5217, (short)0x5223, (short)0x522f, (short)0x523a, (short)0x5246, (short)0x5252, (short)0x525d, (short)0x5269,
			(short)0x5275, (short)0x5280, (short)0x528c, (short)0x5298, (short)0x52a3, (short)0x52af, (short)0x52ba, (short)0x52c6, (short)0x52d2, (short)0x52dd, (short)0x52e9, (short)0x52f4, (short)0x5300, (short)0x530c, (short)0x5317, (short)0x5323,
			(short)0x532e, (short)0x533a, (short)0x5346, (short)0x5351, (short)0x535d, (short)0x5368, (short)0x5374, (short)0x537f, (short)0x538b, (short)0x5396, (short)0x53a2, (short)0x53ae, (short)0x53b9, (short)0x53c5, (short)0x53d0, (short)0x53dc,
			(short)0x53e7, (short)0x53f3, (short)0x53fe, (short)0x540a, (short)0x5415, (short)0x5421, (short)0x542c, (short)0x5438, (short)0x5443, (short)0x544f, (short)0x545a, (short)0x5465, (short)0x5471, (short)0x547c, (short)0x5488, (short)0x5493,
			(short)0x549f, (short)0x54aa, (short)0x54b6, (short)0x54c1, (short)0x54cd, (short)0x54d8, (short)0x54e3, (short)0x54ef, (short)0x54fa, (short)0x5506, (short)0x5511, (short)0x551c, (short)0x5528, (short)0x5533, (short)0x553f, (short)0x554a,
			// 96
			(short)0x5555, (short)0x5561, (short)0x556c, (short)0x5577, (short)0x5583, (short)0x558e, (short)0x559a, (short)0x55a5, (short)0x55b0, (short)0x55bc, (short)0x55c7, (short)0x55d2, (short)0x55de, (short)0x55e9, (short)0x55f4, (short)0x5600,
			(short)0x560b, (short)0x5616, (short)0x5621, (short)0x562d, (short)0x5638, (short)0x5643, (short)0x564f, (short)0x565a, (short)0x5665, (short)0x5670, (short)0x567c, (short)0x5687, (short)0x5692, (short)0x569e, (short)0x56a9, (short)0x56b4,
			(short)0x56bf, (short)0x56cb, (short)0x56d6, (short)0x56e1, (short)0x56ec, (short)0x56f7, (short)0x5703, (short)0x570e, (short)0x5719, (short)0x5724, (short)0x5730, (short)0x573b, (short)0x5746, (short)0x5751, (short)0x575c, (short)0x5768,
			(short)0x5773, (short)0x577e, (short)0x5789, (short)0x5794, (short)0x579f, (short)0x57ab, (short)0x57b6, (short)0x57c1, (short)0x57cc, (short)0x57d7, (short)0x57e2, (short)0x57ed, (short)0x57f9, (short)0x5804, (short)0x580f, (short)0x581a,
			(short)0x5825, (short)0x5830, (short)0x583b, (short)0x5846, (short)0x5851, (short)0x585d, (short)0x5868, (short)0x5873, (short)0x587e, (short)0x5889, (short)0x5894, (short)0x589f, (short)0x58aa, (short)0x58b5, (short)0x58c0, (short)0x58cb,
			(short)0x58d6, (short)0x58e1, (short)0x58ec, (short)0x58f7, (short)0x5902, (short)0x590e, (short)0x5919, (short)0x5924, (short)0x592f, (short)0x593a, (short)0x5945, (short)0x5950, (short)0x595b, (short)0x5966, (short)0x5971, (short)0x597c,
			(short)0x5987, (short)0x5992, (short)0x599d, (short)0x59a8, (short)0x59b2, (short)0x59bd, (short)0x59c8, (short)0x59d3, (short)0x59de, (short)0x59e9, (short)0x59f4, (short)0x59ff, (short)0x5a0a, (short)0x5a15, (short)0x5a20, (short)0x5a2b,
			(short)0x5a36, (short)0x5a41, (short)0x5a4c, (short)0x5a57, (short)0x5a61, (short)0x5a6c, (short)0x5a77, (short)0x5a82, (short)0x5a8d, (short)0x5a98, (short)0x5aa3, (short)0x5aae, (short)0x5ab9, (short)0x5ac3, (short)0x5ace, (short)0x5ad9,
			(short)0x5ae4, (short)0x5aef, (short)0x5afa, (short)0x5b04, (short)0x5b0f, (short)0x5b1a, (short)0x5b25, (short)0x5b30, (short)0x5b3b, (short)0x5b45, (short)0x5b50, (short)0x5b5b, (short)0x5b66, (short)0x5b71, (short)0x5b7b, (short)0x5b86,
			(short)0x5b91, (short)0x5b9c, (short)0x5ba7, (short)0x5bb1, (short)0x5bbc, (short)0x5bc7, (short)0x5bd2, (short)0x5bdc, (short)0x5be7, (short)0x5bf2, (short)0x5bfd, (short)0x5c07, (short)0x5c12, (short)0x5c1d, (short)0x5c28, (short)0x5c32,
			(short)0x5c3d, (short)0x5c48, (short)0x5c53, (short)0x5c5d, (short)0x5c68, (short)0x5c73, (short)0x5c7d, (short)0x5c88, (short)0x5c93, (short)0x5c9e, (short)0x5ca8, (short)0x5cb3, (short)0x5cbe, (short)0x5cc8, (short)0x5cd3, (short)0x5cde,
			(short)0x5ce8, (short)0x5cf3, (short)0x5cfe, (short)0x5d08, (short)0x5d13, (short)0x5d1d, (short)0x5d28, (short)0x5d33, (short)0x5d3d, (short)0x5d48, (short)0x5d53, (short)0x5d5d, (short)0x5d68, (short)0x5d72, (short)0x5d7d, (short)0x5d88,
			(short)0x5d92, (short)0x5d9d, (short)0x5da7, (short)0x5db2, (short)0x5dbd, (short)0x5dc7, (short)0x5dd2, (short)0x5ddc, (short)0x5de7, (short)0x5df1, (short)0x5dfc, (short)0x5e07, (short)0x5e11, (short)0x5e1c, (short)0x5e26, (short)0x5e31,
			(short)0x5e3b, (short)0x5e46, (short)0x5e50, (short)0x5e5b, (short)0x5e65, (short)0x5e70, (short)0x5e7a, (short)0x5e85, (short)0x5e8f, (short)0x5e9a, (short)0x5ea4, (short)0x5eaf, (short)0x5eb9, (short)0x5ec4, (short)0x5ece, (short)0x5ed9,
			(short)0x5ee3, (short)0x5eee, (short)0x5ef8, (short)0x5f03, (short)0x5f0d, (short)0x5f18, (short)0x5f22, (short)0x5f2c, (short)0x5f37, (short)0x5f41, (short)0x5f4c, (short)0x5f56, (short)0x5f61, (short)0x5f6b, (short)0x5f75, (short)0x5f80,
			(short)0x5f8a, (short)0x5f95, (short)0x5f9f, (short)0x5fa9, (short)0x5fb4, (short)0x5fbe, (short)0x5fc9, (short)0x5fd3, (short)0x5fdd, (short)0x5fe8, (short)0x5ff2, (short)0x5ffc, (short)0x6007, (short)0x6011, (short)0x601b, (short)0x6026,
			// 112
			(short)0x6030, (short)0x603b, (short)0x6045, (short)0x604f, (short)0x605a, (short)0x6064, (short)0x606e, (short)0x6078, (short)0x6083, (short)0x608d, (short)0x6097, (short)0x60a2, (short)0x60ac, (short)0x60b6, (short)0x60c1, (short)0x60cb,
			(short)0x60d5, (short)0x60df, (short)0x60ea, (short)0x60f4, (short)0x60fe, (short)0x6108, (short)0x6113, (short)0x611d, (short)0x6127, (short)0x6131, (short)0x613c, (short)0x6146, (short)0x6150, (short)0x615a, (short)0x6165, (short)0x616f,
			(short)0x6179, (short)0x6183, (short)0x618e, (short)0x6198, (short)0x61a2, (short)0x61ac, (short)0x61b6, (short)0x61c1, (short)0x61cb, (short)0x61d5, (short)0x61df, (short)0x61e9, (short)0x61f3, (short)0x61fe, (short)0x6208, (short)0x6212,
			(short)0x621c, (short)0x6226, (short)0x6230, (short)0x623a, (short)0x6245, (short)0x624f, (short)0x6259, (short)0x6263, (short)0x626d, (short)0x6277, (short)0x6281, (short)0x628c, (short)0x6296, (short)0x62a0, (short)0x62aa, (short)0x62b4,
			(short)0x62be, (short)0x62c8, (short)0x62d2, (short)0x62dc, (short)0x62e6, (short)0x62f0, (short)0x62fa, (short)0x6305, (short)0x630f, (short)0x6319, (short)0x6323, (short)0x632d, (short)0x6337, (short)0x6341, (short)0x634b, (short)0x6355,
			(short)0x635f, (short)0x6369, (short)0x6373, (short)0x637d, (short)0x6387, (short)0x6391, (short)0x639b, (short)0x63a5, (short)0x63af, (short)0x63b9, (short)0x63c3, (short)0x63cd, (short)0x63d7, (short)0x63e1, (short)0x63eb, (short)0x63f5,
			(short)0x63ff, (short)0x6409, (short)0x6413, (short)0x641d, (short)0x6427, (short)0x6431, (short)0x643b, (short)0x6445, (short)0x644f, (short)0x6459, (short)0x6462, (short)0x646c, (short)0x6476, (short)0x6480, (short)0x648a, (short)0x6494,
			(short)0x649e, (short)0x64a8, (short)0x64b2, (short)0x64bc, (short)0x64c6, (short)0x64cf, (short)0x64d9, (short)0x64e3, (short)0x64ed, (short)0x64f7, (short)0x6501, (short)0x650b, (short)0x6515, (short)0x651e, (short)0x6528, (short)0x6532,
			(short)0x653c, (short)0x6546, (short)0x6550, (short)0x655a, (short)0x6563, (short)0x656d, (short)0x6577, (short)0x6581, (short)0x658b, (short)0x6595, (short)0x659e, (short)0x65a8, (short)0x65b2, (short)0x65bc, (short)0x65c6, (short)0x65cf,
			(short)0x65d9, (short)0x65e3, (short)0x65ed, (short)0x65f6, (short)0x6600, (short)0x660a, (short)0x6614, (short)0x661e, (short)0x6627, (short)0x6631, (short)0x663b, (short)0x6645, (short)0x664e, (short)0x6658, (short)0x6662, (short)0x666b,
			(short)0x6675, (short)0x667f, (short)0x6689, (short)0x6692, (short)0x669c, (short)0x66a6, (short)0x66b0, (short)0x66b9, (short)0x66c3, (short)0x66cd, (short)0x66d6, (short)0x66e0, (short)0x66ea, (short)0x66f3, (short)0x66fd, (short)0x6707,
			(short)0x6710, (short)0x671a, (short)0x6724, (short)0x672d, (short)0x6737, (short)0x6741, (short)0x674a, (short)0x6754, (short)0x675e, (short)0x6767, (short)0x6771, (short)0x677a, (short)0x6784, (short)0x678e, (short)0x6797, (short)0x67a1,
			(short)0x67ab, (short)0x67b4, (short)0x67be, (short)0x67c7, (short)0x67d1, (short)0x67db, (short)0x67e4, (short)0x67ee, (short)0x67f7, (short)0x6801, (short)0x680a, (short)0x6814, (short)0x681e, (short)0x6827, (short)0x6831, (short)0x683a,
			(short)0x6844, (short)0x684d, (short)0x6857, (short)0x6860, (short)0x686a, (short)0x6874, (short)0x687d, (short)0x6887, (short)0x6890, (short)0x689a, (short)0x68a3, (short)0x68ad, (short)0x68b6, (short)0x68c0, (short)0x68c9, (short)0x68d3,
			(short)0x68dc, (short)0x68e6, (short)0x68ef, (short)0x68f9, (short)0x6902, (short)0x690c, (short)0x6915, (short)0x691e, (short)0x6928, (short)0x6931, (short)0x693b, (short)0x6944, (short)0x694e, (short)0x6957, (short)0x6961, (short)0x696a,
			(short)0x6973, (short)0x697d, (short)0x6986, (short)0x6990, (short)0x6999, (short)0x69a3, (short)0x69ac, (short)0x69b5, (short)0x69bf, (short)0x69c8, (short)0x69d2, (short)0x69db, (short)0x69e4, (short)0x69ee, (short)0x69f7, (short)0x6a01
	};

	/**
	 * Used to convert a sin value from (0*256 - 1*256) into a cosine value.
	 *
	 * There are 128 values in this table representing sin values increasing in
	 * increments of 1/128.
	 *
	 * Each value is calculated by getting the inverse sin of the sin value, to
	 * give the actual angle, then taking the cosine of this angle.  The result
	 * is then multiplied by 256.
	 *
	 * First 8 values should ideally be 256.
	 */
	static final byte[] b_cosine_conversion_table_fp8 = {
			(byte)0xff, (byte)0xff, (byte)0xff, (byte)0xff, (byte)0xff, (byte)0xff, (byte)0xff, (byte)0xff,
			(byte)0xff, (byte)0xff, (byte)0xff, (byte)0xff, (byte)0xff, (byte)0xff, (byte)0xfe, (byte)0xfe,
			(byte)0xfe, (byte)0xfe, (byte)0xfd, (byte)0xfd, (byte)0xfd, (byte)0xfd, (byte)0xfc, (byte)0xfc,
			(byte)0xfb, (byte)0xfb, (byte)0xfb, (byte)0xfa, (byte)0xfa, (byte)0xf9, (byte)0xf9, (byte)0xf8,
			(byte)0xf8, (byte)0xf7, (byte)0xf7, (byte)0xf6, (byte)0xf6, (byte)0xf5, (byte)0xf4, (byte)0xf4,
			(byte)0xf3, (byte)0xf3, (byte)0xf2, (byte)0xf1, (byte)0xf0, (byte)0xf0, (byte)0xef, (byte)0xee,
			(byte)0xed, (byte)0xec, (byte)0xec, (byte)0xeb, (byte)0xea, (byte)0xe9, (byte)0xe8, (byte)0xe7,
			(byte)0xe6, (byte)0xe5, (byte)0xe4, (byte)0xe3, (byte)0xe2, (byte)0xe1, (byte)0xe0, (byte)0xdf,
			(byte)0xde, (byte)0xdd, (byte)0xdb, (byte)0xda, (byte)0xd9, (byte)0xd8, (byte)0xd6, (byte)0xd5,
			(byte)0xd4, (byte)0xd2, (byte)0xd1, (byte)0xcf, (byte)0xce, (byte)0xcc, (byte)0xcb, (byte)0xc9,
			(byte)0xc8, (byte)0xc6, (byte)0xc5, (byte)0xc3, (byte)0xc1, (byte)0xbf, (byte)0xbe, (byte)0xbc,
			(byte)0xba, (byte)0xb8, (byte)0xb6, (byte)0xb4, (byte)0xb2, (byte)0xb0, (byte)0xae, (byte)0xac,
			(byte)0xa9, (byte)0xa7, (byte)0xa5, (byte)0xa2, (byte)0xa0, (byte)0x9d, (byte)0x9b, (byte)0x98,
			(byte)0x95, (byte)0x92, (byte)0x8f, (byte)0x8c, (byte)0x89, (byte)0x86, (byte)0x83, (byte)0x7f,
			(byte)0x7c, (byte)0x78, (byte)0x74, (byte)0x70, (byte)0x6c, (byte)0x68, (byte)0x63, (byte)0x5e,
			(byte)0x59, (byte)0x53, (byte)0x4d, (byte)0x47, (byte)0x3f, (byte)0x37, (byte)0x2d, (byte)0x20
		};

	static final short[] w_maximum_angles = {
			44 * 256, 10 * 256,
			-45 * 256, -11 * 256
		};
	
	// get_sin_cos
	/**
	 * Get sin or cos. Fixed point. 1 is 32768. 0.707 is 23170
	 *
	 * @param angle angle as word. d0.
	 * @param shift as word. d5.
	 *          0 - the result is sin,
	 *          {@link #ANGLE_90_DEGREES} - the result is cos
	 * @return d0 as word. 0...32767.
	 */
	static short getSinCos(int angle, final int shift) {
		int quarter = angle & ANGLE_90_DEGREE_MASK;// 0x3FFF
		if( 0 == ((angle & ANGLE_90_DEGREES) ^ shift) ) {// 0x4000
			quarter ^= 0x3FFF;// d3 = -d3 - 1
			++quarter;// d3 = -d3, 0 ... 0x4000
		}
		final int fraction = (quarter >>> 6) & 0x1FF;//  0...1FF. 90/64 ~= 1 deg
		int d6 = (w_cos_table[ fraction ] - w_cos_table[ fraction + 1 ]);// cos( a ) - cos( a + 90/256 )
		d6 *= (quarter << 10) & 0xfc00;
		d6 >>>= 16;
		int d7 = ((w_cos_table[ fraction ] - d6) & 0xFFFF) >>> 1;// cos( a )
		quarter = (angle & shift) << 1;
		if( 0 != ((angle ^ quarter) & 0x8000)  ) {
			d7 = -d7;
		}
		return (short) d7;
	}

	// get_sin
	static short getSin(final int d0) {
		return getSinCos( d0, 0 );
	}

	// get_cos
	static short getCos(final int d0) {
		return getSinCos( d0, ANGLE_90_DEGREES );
	}

	// calculate_perspective_coord
	/**
	 * in: {@link #w_perspective_table} is used
	 *
	 * @param sd0_x X coordinate, as word
	 * @param sd3_z Z coordinate, as word
	 * @return (d7 as word << 48) | (d5 as word << 32) | (d4 as word << 16) | (d0 as word)
	 */
	static long calculatePerspectiveCoord(int sd0_x, int sd3_z) {
		int sd4 = sd0_x;
		if( sd4 < 0 ) {
			sd0_x = -sd0_x;
		}
		int sd5 = sd3_z;
		if( sd5 < 0 ) {
			sd3_z = -sd3_z;
		}
		final int sd7;
		if( sd3_z == sd0_x ) {// |x| == |z|
			sd7 = 0xffff;
			sd0_x = 0x2000;
		} else {
			if( sd3_z <= sd0_x ) {// |z| <= |x|
				sd3_z <<= 16;
				sd3_z /= sd0_x;// unsigned
				sd7 = sd3_z;// |z| / |x| .fp16
				sd3_z >>>= 5;// |z|.fp11
				sd0_x = w_perspective_table[ sd3_z ];
				if( (sd4 ^ sd5) >= 0 ) {// sign x == sign z
					sd0_x = -sd0_x;
				}
				sd3_z = 0x4000;// +16384
				if( sd4 < 0 ) {// x
					sd3_z = -0x4000;// -16384
				}
				sd0_x += sd3_z;
				return (((long)sd7) << 48)
						| ((((long)sd5) & 0xFFFF) << 32)
						| ((((long)sd4) & 0xFFFF) << 16)
						| (((long)sd0_x) & 0xFFFF);
			}
			sd0_x <<= 16;
			sd0_x /= sd3_z;// unsigned
			sd7 = sd0_x;
			sd0_x = w_perspective_table[ sd0_x >>> 5 ];
		}
		sd3_z = sd4 ^ sd5;
		if( sd3_z < 0 ) {
			sd0_x = -sd0_x;
		}
		if( sd5 < 0 ) {
			sd0_x += 0x8000;// +32768
		}
		return (((long)sd7) << 48)
				| ((((long)sd5) & 0xFFFF) << 32)
				| ((((long)sd4) & 0xFFFF) << 16)
				| (((long)sd0_x) & 0xFFFF);
	}

	// calculate_perspective_value
	/**
	 *
	 * @param r (d7 as word << 48) | (d5 as word << 32) | (d4 as word << 16) | (d0 as word)
	 * @return d0 as word
	 */
	static int calculatePerspectiveValue(final long r) {
		int sd7 = (int)(short)(r >>> 48);
		int sd5 = (int)(short)(r >>> 32);
		int sd4 = (int)(short)(r >>> 16);
		if( sd4 < 0 ) {
			sd4 = -sd4;
		}
		if( sd5 < 0 ) {
			sd5 = -sd5;
		}
		if( sd5 < sd4 ) {
			final int temp = sd4;
			sd4 = sd5;
			sd5 = temp;
		}
		return ((sd4 * w_perspective_table_2[ ((sd7 & 0xFFFE) >>> 4) >> 1 ]) >> 16) + sd5;
	}
}
